#!/usr/bin/env perl
###############################################################################
#    Webmin Sysstats - batch_graph.cgi
#
#    Stats Webmin Module
#    Copyright (C) 2003 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this program is designed to produce batch graphs (for exemple by cron)
# rem : the code is very similar to batch_install.pl
###############################################################################

use strict;
use warnings;
use File::Basename;
use Getopt::Long;
use Pod::Usage;
use Cwd 'abs_path';

use English qw(-no_match_vars);

##################################################################################
sub init_batch() {
	my ( $scriptname, $sysstats_dir, undef ) =
	  fileparse( abs_path $PROGRAM_NAME);
	chdir $sysstats_dir or die "can not chdir to $sysstats_dir: $ERRNO\n";

	# sysstats_dir should be something like /usr/libexec/webmin/sysstats
	push @INC, $sysstats_dir;         # webminstats dir
	push @INC, "$sysstats_dir/..";    # webmin dir

	my $webmin_module = basename($sysstats_dir);    # should be 'sysstats'

	use subs qw( &read_file &text );                # from web-lib.pl
	use subs qw( &graph_data );                     # from sysstats-lib.pl

	## no critic (ProhibitPackageVars)
	use vars
	  qw( %gconfig %config %text $module_config_directory $root_directory $module_name);

	use vars qw( $module_dir );                     # from sysstats-lib.pl

	# prepare for init_config
	# no critic (ProhibitNoWarnings)
	no warnings;
	$main::initial_process_id = $PID;
	use warnings;
	## use critic
	$ENV{'WEBMIN_CONFIG'} = '/etc/webmin' if ( !exists $ENV{'WEBMIN_CONFIG'} );
	$ENV{'WEBMIN_VAR'}    = '/var/webmin' if ( !exists $ENV{'WEBMIN_VAR'} );
	$ENV{'SCRIPT_NAME'} = "$webmin_module/$scriptname";
	$ENV{'SERVER_ROOT'} = $sysstats_dir;
	$ENV{'BATCH_MODE'}  = 'y';

	# call init_config
	## no critic (RequireBarewordIncludes)
	require 'sysstats-lib.pl';
	## use critic

	# fix some others vars after init_config
	$module_name             = $webmin_module;
	$module_config_directory = "$ENV{'WEBMIN_CONFIG'}/$module_name";
	$root_directory          = dirname($sysstats_dir);

	# read config files
	batch_readconf();

	return;
}

###############################################################################
# main
###############################################################################

init_batch();

# variables for program option parsing
my $module;
my $working_dir;
my $period;
my $end;
my $display;
my $help;
my $man;

## no critic (ProhibitUnlessBlocks)
unless (
	GetOptions(
		'module|m=s'         => \$module,
		'working_dir|d=s'    => \$working_dir,
		'period|p=s'         => \$period,
		'end|e=s'            => \$end,
		'display_number|n=i' => \$display,
		'help'               => \$help,
		'man'                => \$man,
	)
  )
{
	pod2usage('bad syntax');
}
## use critic

# check options
if ($help) {
	pod2usage(1);
}
elsif ($man) {
	pod2usage( -verbose => 2 );
}
elsif ( !defined $module ) {

	# "old syntaxe"
	# argument are at fixed positions
	$module      = $ARGV[0];
	$working_dir = $ARGV[1];
	$display     = $ARGV[2];
}

# controls
if ( !$module ) {
	pod2usage('no module name');
}

# we have a module name as cpu,
my $fmodule = "$module_dir/$module";

# should be a directory
if ( !-d $fmodule ) {
	die "$fmodule is not a directory : not a webminstats module\n";
}

my $opt = undef;

# Read module information to find whitch module we are using

my ( %my_config, %defaults );
my %minfo = read_module_conf( $module, \%text, \%my_config, \%defaults );

if ( $minfo{'module_multi_display'} ) {

	# need a display
	if ( defined $display ) {
		$opt = 'display' . $display;
	}
	else {
		die "module $module needs a display\n";
	}
}
elsif ( defined $display ) {
	$ENV{'display'} = $display;
}

# a customized end ?
if ( defined $end ) {
	my $s_end = period2sec($end);
	if ($s_end) {

		# 0 means now, negative for the past
		$minfo{'custom_end'} = -$s_end;
	}
	else {
		pod2usage('bad end parameter');
	}
}

# a customized period ?
if ( defined $period ) {
	my $s_period = period2sec($period);
	if ($s_period) {
		$minfo{'custom_period'} = $s_period;
	}
	else {
		pod2usage('bad period parameter');
	}
}

# second arg : directory to put graphs
$working_dir = $working_dir || '/tmp';

$minfo{'tmpgname'} = "$working_dir/$module";

require $minfo{'mod_lib'};

# call sub in module lib
if ($opt) {
	graph_data( \%minfo, \%my_config, $opt );
}
else {
	graph_data( \%minfo, \%my_config );
}

# search for results
my @res = glob "$working_dir/$module*";

print "result(s) for $module are on $working_dir directory : @res\n";

__END__

=head1 NAME

batch_graph.pl

=head1 DESCRIPTION

this tool will produce graphs from webminstats database (rrdtool) in batch mode

=head1 SYNOPSIS

new syntaxe : batch_graph.pl --module module --working_dir working_directory --display_number display_number --period delai --end delai

old syntaxe : batch_graph.pl module working_directory display_number

=head1 USAGE

batch_graph.pl --module cpu --working_dir ~/tmp --period 1W

=head1 REQUIRED ARGUMENTS

the module name

=head1 OPTIONS

=over 4

=item module

can be specified with name only : for example cpu

=item working_directory

is the location where the graphics are created (/tmp by default)

=item display_number

is a numerical needed for modules such network to select a component, for example 2

=item period

is optional and is used to fix the time scale (not possible in old syntaxe)
the form is xP, x un number, P is H for hours, D for days, W for weeks, M for months : 2W ask for 2 weeks

=item end

is optional and is used to fix the end date of the graph (show graph in the past)
it uses the same form as the period parameter

=back

=head1 DIAGNOSTICS

depends if run in debug mode or not

=head1 EXIT STATUS

0 if ok

=head1 CONFIGURATION

it depends upon webmin configuration (/etc/webmin/config) and webminstats configuration (/etc/webmin/sysstats/config)

=head1 DEPENDENCIES

webmin code (and lib) should be installed

=head1 INCOMPATIBILITIES

not known

=head1 BUGS AND LIMITATIONS

not known yet

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2006 by Eric Gerbier
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

=head1 AUTHOR

Eric Gerbier

you can report any bug or suggest to gerbier@users.sourceforge.net

=cut

