#!/usr/bin/perl -w
###############################################################################
#    Webmin Syststats - index.cgi
#
#    Stats Webmin Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

use strict;
use warnings;
use English qw(-no_match_vars);

## no critic (RequireBarewordIncludes)
require './sysstats-lib.pl';
## use critic

# from web-lib.pl
## no critic (ProhibitPackageVars)
use vars qw( %gconfig %config %text $module_config_directory %in);
use subs
  qw( &ui_print_header &ui_print_footer &error &text_error &write_file &read_file );

# from sysstats-lib.pl
use vars qw( %access $cron_dir $module_dir );
use subs qw( &debug_sub_begin &debug_sub_end &write_file_hash);
## use critic

###############################################################################
# Verify the rrdtool installed
# check for 'rrdtool' in config file or in path
# check for RRDs.pm (perl module)
sub check_rrdtool () {
	debug_sub_begin();

	# test for rrdtool command
	if ( !-x $config{'rrdtool'} ) {

		# not in config :  search in PATH
		my $rrdtool = search_command('rrdtool');
		if ( defined $rrdtool ) {
			$config{'rrdtool'} = $rrdtool;
			write_file_hash( $module_config_directory . '/config', \%config );
		}
		else {
			error(
				text(
					'index_epath', $config{'rrdtool'},
					'../config.cgi?sysstats'
				)
			);
		}
	}
	elsif ( !-f $config{'rrdtool'} ) {

		# in config, but not a file
		# is this a bad config with only a path to the directory ?
		my $rrdtool = $config{'rrdtool'} . '/rrdtool';
		if ( -x $rrdtool ) {
			$config{'rrdtool'} = $rrdtool;
			write_file_hash( $module_config_directory . '/config', \%config );
		}
		else {
			error(
				text(
					'index_epath', $config{'rrdtool'},
					'../config.cgi?sysstats'
				)
			);
		}
	}

	# test for perl module : RRDs
	# not useful here, because require chain is
	# index.cgi -> sysstats-lib -> sysstats-rrd-lib -> require RRDs

	debug_sub_end();
	return;
}
###################################################################################################
# display main page
###################################################################################################
# display webminstats status (run/stop) and if allowed the button to change it
sub display_runstop_status () {
	debug_sub_begin();

	print ui_hr();

	# display last aquisition run
	my $lastrun = get_lastrun();
	my $curtime = time;
	my ( $color, $txt );
	if ($lastrun) {
		$txt = strftime( '%Y/%m/%d %H:%M:%S', localtime $lastrun );
		$color = ( ( $curtime - $lastrun ) < 120 ) ? COLOR_OK() : 'red';
	}
	else {
		$color = 'red';
		$txt   = 'unknown';
	}
	print $text{'index_last_aquisition'}
	  . html_color( $color, $txt )
	  . '<br />';

	# lock file
	my $date_lock = is_locked();
	if ($date_lock) {
		$txt = strftime( '%Y/%m/%d %H:%M:%S', localtime $date_lock );
		$color = ( ( $curtime - $date_lock ) < 60 ) ? COLOR_OK() : 'red';
		print $text{'index_aquisition_lock'}
		  . html_link( 'clear_lock.cgi', html_color( $color, $txt ) )
		  . $text{'index_clear_lock'}
		  . '<br />';
	}

	if ( $access{'runstop'} ) {

		# can change state : show buttons
		print wrapper( 'ui_form_start', 'startstop.cgi' );
		if ( is_webmin_sysstats_running() ) {

			# stop button
			print wrapper( 'ui_hidden', 'idx', 'stop' );
			print wrapper( 'ui_form_end', [ [ undef, $text{'index_stop'} ] ] )
			  . $text{'index_stopmsg'};
		}
		else {

			# start button
			print wrapper( 'ui_hidden', 'idx', 'start' );
			print wrapper( 'ui_form_end', [ [ undef, $text{'index_start'} ] ] )
			  . $text{'index_startmsg'};
		}
	}
	else {

		# just display status
		print ui_hr();
		my $status =
		  ( is_webmin_sysstats_running() )
		  ? $text{'running'}
		  : $text{'stopped'};
		print "aquisition status : $status<br>\n";
	}

	print ui_hr();

	debug_sub_end();
	return;
}
###################################################################################################
sub button_update() {

	# a button to force full check (2)
	print wrapper( 'ui_form_start', 'index.cgi' );
	print wrapper( 'ui_hidden', 'force_check', 2 );
	print wrapper( 'ui_form_end', [ [ undef, $text{'check_update'} ] ] );

	return;
}
###################################################################################################
# display current version
# and show updates if configured for
sub display_version(@) {
	my @mod_list = @_;
	debug_sub_begin();

	# display webminstats version
	my $version = get_sysstats_version();
	my $lang    = get_lang();
	my $theme   = get_gconfig('theme') || q{};

	print
"webminstats version $version<br>os $gconfig{real_os_type} $gconfig{real_os_version}<br>language $lang<br>theme $theme<br>\n";

	# release control
	if ( ( $config{'check_update'} ) || ( exists $in{'force_check'} ) ) {

		my %versions;

		# main
		$versions{'webminstats'} = $version;

		# modules
		foreach my $mod (@mod_list) {
			$versions{ $mod->{'module_name'} } = $mod->{'module_ver'};
		}

		my $level =
		  ( exists $in{'force_check'} )
		  ? $in{'force_check'}
		  : $config{'check_update'};
		if ( check_update( \%versions, $level ) ) {

			# some changes were found
			# the button will allow to display details
			button_update();
		}
	}
	else {
		button_update();
	}
	print "<hr>\n";

	debug_sub_end();
	return;
}
###################################################################################################
# display icons of webminstats modules
sub display_main () {
	debug_sub_begin();
	my @mod_list = get_mod_list(0);    #all modules

	# running modules
	my ( @rlinks, @rtitles, @ricons );

	# stopped modules
	my ( @slinks, @stitles, @sicons );

	foreach my $mod (@mod_list) {
		my ( %my_config, %my_text );

		if ( !read_file( $mod->{'mod_lang'}, \%my_text ) ) {
			print text( 'ecannot_readlang', $mod->{'mod_lang'}, "\n" );
			next;
		}

		if ( !read_module_config( \%my_config, $mod->{'config_file'} ) ) {
			print '<tt><p>'
			  . text( 'ecannot_readconfig', $mod->{'config_file'} )
			  . "\n</p></tt>";
			next;
		}

		# check for module controled display
		if ( $mod->{'module_multi_display'} ) {
			my $i = 1;
			while ( exists $my_config{ 'display' . $i } ) {
				my ( $net_num, $sub_display, $title_idx ) =
				  split /,/, $my_config{ 'display' . $i };
				push @rtitles, $my_config{ 'module_index_title' . $title_idx };
				push @rlinks,
				  'display.cgi?idx=' . $mod->{'idx'} . '&opt=display' . $i;
				push @ricons, $mod->{'mod_dir'} . $mod->{'module_icon'};

				$i++;
			}
		}
		else {
			my ( $r_titles, $r_icons );    # ref to real data
			if ( mod_check_runstop( $mod, \%my_config ) ) {

				#stopped
				$r_titles = \@stitles;
				$r_icons  = \@sicons;

				push @slinks, 'edit_module.cgi?idx=' . $mod->{'idx'};
			}
			else {

				# running modules
				$r_titles = \@rtitles;
				$r_icons  = \@ricons;
				push @rlinks, 'display.cgi?idx=' . $mod->{'idx'};
			}
			if ( $my_config{'module_title_index'} ) {
				push @{$r_titles}, $my_config{'module_title_index'};
			}
			else {
				push @{$r_titles}, $my_text{'module_title_index'};
			}

			push @{$r_icons}, $mod->{'mod_dir'} . $mod->{'module_icon'};
		}
	}

	print "<hr>\n";

	# Display running modules icons
	print ui_subheading( $text{'menu_running_modules'} );
	my_icons_table( \@rlinks, \@rtitles, \@ricons, $config{'icon_by_row'} );

	print "<hr>\n";

	# Display stopped  modules icons
	print ui_subheading( $text{'menu_stopped_modules'} );
	my_icons_table( \@slinks, \@stitles, \@sicons, $config{'icon_by_row'} );

	# same footer icons as others screen
	my_footer();

	display_runstop_status();

	display_version(@mod_list);

	wrapper( 'ui_print_footer', SLASH(), "webmin $text{'index'}" );
	debug_sub_end();
	return;
}

#################################################################################################################
# 					main
#################################################################################################################

debug_sub_begin();
wrapper('ReadParse');

# check if rrdtool is available
check_rrdtool();

# Display page header
wrapper(
	'ui_print_header',
	undef,
	$text{'title_index'},
	undef, 'intro', 1, 1, 0,
	$text{index_written} . q{ }
	  . html_link( 'mailto:gerbier@users.sourceforge.net', 'Eric Gerbier' )
	  . '<br>'
	  . html_link(
		'http://webminstats.sourceforge.net', $text{'index_homepage'}
	  )
);

# to install/upgrade modules
check_new_modules();

# main part : display icons
display_main();

# end
debug_sub_end();
