
###############################################################################
#    Stats Webmin Module
#    Copyright (C) 2002 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this lib is used by all aquisition scripts (launched by cron)
# it depens upon the general lib sysstats-gen-lib.pl
###############################################################################
# see also config_action.cgi

use strict;
use warnings;
use Cwd;
use File::Basename;
use English qw(-no_match_vars);

# is set in path by the caller with a use lib
## no critic (RequireBarewordIncludes)
require 'sysstats-gen-lib.pl';
require 'sysstats-alarm-lib.pl';

# from web-lib.pl
## no critic (ProhibitPackageVars)
use vars
  qw( %gconfig %config %text $module_config_directory $cb $tb %in %access %tconfig );

## use critic

# flag file to avoid spam on mailto action
# work as alarms
my $Flag = 'action_';

# required tools
my %pre_req = (
	mailto => '/bin/mail',
	top    => '/usr/bin/top',
	who    => '/usr/bin/w',
);

# action list
{
	my %actions = (
		mail   => \&action_mail,
		mailto => \&action_mailto,
		run    => \&action_exec,
		top    => \&action_top,
		who    => \&action_who,
		log    => \&action_log,
	);

	sub get_actions() {
		return %actions;
	}

	sub get_list_actions() {
		return keys %actions;
	}

	# clean action if pre-requisite are not available
	sub check_prereq() {

		foreach my $key ( keys %pre_req ) {
			if ( !-e $pre_req{$key} ) {
				delete $actions{$key};
			}

		}
		return;
	}
}
############################################################
# return content of cuurent action
# text is saved in Flag file
sub is_cur_action($) {
	my $dir = shift @_;

	my @list = glob $dir . $Flag . q{*};
	my $txt  = q{};
	foreach my $fic (@list) {
		my $r_tab = read_full_file($fic);
		foreach my $ligne ( @{$r_tab} ) {
			chomp $ligne;
			$txt .= $ligne . q{;};
		}
	}
	return $txt;
}
############################################################
sub action_is_action_key($) {
	my $key = shift @_;

	return ( $key =~ m/^action\d+$/ );
}
############################################################
# extract code from config file
sub action_load($) {
	my $r_config = shift @_;

	my %codes;
	foreach my $key ( sort keys %{$r_config} ) {
		if ( action_is_action_key($key) ) {
			$codes{$key} = $r_config->{$key};
		}
	}
	return %codes;
}
############################################################
# will return number of actions for the given config
sub is_action($) {
	my $r_config = shift @_;

	my %codes = action_load($r_config);
	return scalar keys %codes;
}
############################################################
# return first available key for config
sub action_first_free_key($) {
	my $rh_codes = shift @_;

	my $index = -1;
  CODE: foreach my $key ( sort keys %{$rh_codes} ) {
		if ( $key =~ m/(\d+)/ ) {
			( undef, $index ) = $1;
		}
	}
	$index++;
	return $index;
}

############################################################
sub test_flag($) {
	my $flag_file = shift @_;

	my $do_action = 0;
	if ( -f $flag_file ) {

		# we have an old flag
		my $max = $config{'action_freq'} || 3_600;
		if ( file_age_s($flag_file) > $max ) {

			# too old action
			$do_action = 1;
		}
	}
	else {
		# first action for this rule
		$do_action = 1;
	}
	return $do_action;
}
############################################################
# execute code
sub action_param($$$) {
	my $real_param = shift @_;    # name
	my $real_val   = shift @_;    # value
	my $rh_codes   = shift @_;    # action configuration

	my %actions = get_actions();

  CODE: foreach my $key ( sort keys %{$rh_codes} ) {
		my $conf = $rh_codes->{$key};

		my ( $conf_param, $conf_op, $conf_val, $action, $arg, $active ) =
		  action_decode($conf);

		# one flag file by rule
		my $flag_file = $Flag . $key;

		if ( !$active ) {

			# clear flag if exists
			if ( -f $flag_file ) {
				unlink $flag_file;
				my $txt = "module=$ENV{'module_name'} key=$key action=$action";
				$txt .= " arg=$arg" if ( defined $arg );
				journaliser( action_journal(), "disable $txt" );
			}
			next CODE;
		}

		if (    ( $real_param eq $conf_param )
			and ( defined $real_val )
			and ( $real_val ne q{U} )
			and ( $real_val ne q{} ) )
		{
			## no critic (ProhibitStringyEval)
			my $exp  = "$real_val $conf_op $conf_val";
			my $tst  = eval "$exp";
			my $cond = "$real_param = $exp";
			my $txt =
			  "module=$ENV{'module_name'} key=$key cond=$cond action=$action";
			$txt .= " arg=$arg" if ( defined $arg );
			## use critic
			if ($EVAL_ERROR) {
				warning("problem in code $key ($exp): $EVAL_ERROR");
				next CODE;
			}
			elsif ($tst) {

				# test is true
				if ( test_flag($flag_file) ) {

					# the test is ok : will have to start defined action
					write_file_txt( $flag_file, $cond );

					$actions{$action}( $arg, $cond );

					# journalisation only if action run
					journaliser( action_journal(), $txt );
				}
			}
			else {
				# test is false : clean flag file
				if ( -f $flag_file ) {
					unlink $flag_file;

					# journaliser : with action_freq period, we miss actions
					# and we do not know when it end
					journaliser( action_journal(), "recovery $txt" );
				}
			}
		}    # param
	}    # foreach
	return;
}
############################################################
# code to executed
############################################################
# start who command
# result send to output => mail
sub action_who($$) {

	# do not use parameter

	my $cmd = $pre_req{'who'};
	_action_exec($cmd);

	return;
}
############################################################
# start top command
# result send to output => mail
sub action_top($$) {

	# do not use parameter

	my $cmd = "$pre_req{'top'} -n 1 -b";
	_action_exec($cmd);

	return;
}
############################################################
# mail action just use warning, the cron job will send the mail
# to root or to another address if set in webminstats config
sub action_mail($$) {
	my $arg  = shift @_;    # action argument
	my $cond = shift @_;    # test condition

	# display arg if defined else condition
	my $msg = ( ( !$arg ) or ( $arg eq SPACE() ) ) ? $cond : $arg;
	warning($msg);
	return;
}
############################################################
# the difference with action_mail is that we want to define
# a mail address for each test.
# !! this do not use warning output function,
# to avoid email spam (one by minute) use the same technic as alarms
# with a a flag file
# if exists flag
# then if age > x
# 	then old status : remind
# 	else recent : skip
# else
# 	new mail
#
sub action_mailto($$) {
	my $arg  = shift @_;    # action argument : dest mail
	my $cond = shift @_;    # test condition

	# to send a mail , do not use any perl module, use mail command
	my $subject = "\'webminstats alarm on $ENV{'module_name'}\'";
	my $cmd     = "$pre_req{'mailto'} -s $subject $arg";
	## no critic (ProhibitTwoArgOpen)
	if ( open my $fh, " | $cmd " ) {
		print {$fh} $cond;
		close $fh or warning("can not close mail : $ERRNO");
	}
	else {
		warning("can not send mail : $ERRNO");
	}
	return;
}
############################################################
# start any defined command
sub action_run($$) {
	my $cmd  = shift @_;    # command to be runned
	my $cond = shift @_;    # test condition

	_action_exec($cmd);
	return;
}
############################################################
sub _action_exec($) {
	my $cmd = shift @_;

	## no critic (ProhibitBacktickOperators)
	my $out = `$cmd 2>&1`;
	## use critic
	warning($out);
	return;
}
############################################################
# write in a custom log file
sub action_log($$) {
	my $log  = shift @_;    # log file (from arg)
	my $cond = shift @_;    # test condition

	if ( !$log ) {

		# do nothing
		# write to journal is done in action_param sub
	}
	elsif ( open my $fh, '>>', $log ) {

		my $ltime = logdate();
		my $txt   = "$ENV{'module_name'} $cond";
		print {$fh} "$ltime $txt\n";

		## no critic (RequireCheckedSyscalls,RequireCheckedClose)
		close $fh;
	}
	else {
		warning("can not write to log file $log : $ERRNO");
	}
	return;
}
############################################################
# end of code section
############################################################
my $action_sep = COMMA();
############################################################
sub action_encode($$$$$) {
	my @tab = @_;

	my $line = join $action_sep, @tab;
	return $line;
}
############################################################
sub action_decode($) {
	my $line = shift @_;

	my @tab = split $action_sep, $line;

	# add active parameter if not exists
	my $nb_params = scalar @tab;
	if ( $nb_params == 5 ) {
		push @tab, 'on';
	}

	return @tab;
}
############################################################
touch( action_journal() );
check_prereq();
1;
