
###############################################################################
#    Stats Webmin Module
#    Copyright (C) 2002 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this lib is the messaging system too avoid too many mails when a
# problem is detected on aquisition
# the code is used in aquisition and in core (web)
###############################################################################

use strict;
use warnings;
use Cwd;
use File::Basename;
use English qw(-no_match_vars);
use POSIX qw( strftime );

# is set in path by the caller with a use lib
## no critic (RequireBarewordIncludes)
require 'sysstats-constants-lib.pl';

# from web-lib.pl
## no critic (ProhibitPackageVars)
use vars
  qw( %gconfig %config %text $module_config_directory $cb $tb %in %access %tconfig );

## use critic

# flag for permanent alarm (from previous run)
my $alarm = 'alarm';

# flag for current run
my $cur_alarm = 'cur_alarm';

my $localdir = q{./};

#####################################################################################
# we set a file flag in the module directory when the problem is detected
# and we remove it if the problem is cleared
# nb : alarms are done by module

#####################################
# return true if the previous run sent an alarm
sub test_perm_alarm(;$) {
	my $dir = shift @_ || $localdir;

	my $f_alarm = $dir . $alarm;
	return ( -f $f_alarm );
}

#--------------------------------------
# transform a current alarm to permanent
sub _set_perm_alarm() {
	rename $cur_alarm, $alarm;
	return;
}

#--------------------------------------
# clean permanent alarm
sub _reset_perm_alarm(;$) {
	my $dir = shift @_ || $localdir;

	unlink $dir . $alarm;
	return;
}

#####################################
# return true if the curret run sent an alarm
sub _test_cur_alarm() {
	return -f $cur_alarm;
}

#--------------------------------------
# clean current alarm
sub _reset_cur_alarm() {
	unlink $cur_alarm;
	return;
}

#--------------------------------------
# register alarms messages in the file
sub set_cur_alarm(@) {

	# open new file or append at end of file
	my $mode = _test_cur_alarm() ? '>>' : '>';
	if ( open my $fh, $mode, $cur_alarm ) {
		if ( $mode eq '>' ) {

			# first line will be the date of first alarm
			print {$fh} strftime( '%Y/%m/%d %H:%M:%S', localtime ) . "\n";
		}
		foreach my $ligne (@_) {
			print {$fh} $ligne . "\n";
		}

		# because we are already in warning system
		## no critic (RequireCheckedSyscalls,RequireCheckedClose)
		close $fh;
		## use critic
	}
	return;
}

#####################################################################################
# to be called at end of each aquisition script (by sysstats.pl)
# to reset the permanent flags if necessary
sub analyse_alarm($) {
	my $module_name = shift @_;

	if ( _test_cur_alarm() ) {

		# we had an alarm in the current run
		if ( test_perm_alarm() ) {

			# it is not a new one : remove it
			_reset_cur_alarm();

			# reminder : send a mail once a day if alarm persists
			my $age = file_age_s($alarm);
			my $max = $config{'alarm_freq'} || 86_400;
			if ( $age >= $max ) {
				warn "reminder old alarm on module $module_name "
				  . get_alarm_msg() . "\n";

				# then change date of change to avoid alarm each minute
				touch($alarm);
			}
		}
		else {

			# it is a new one : keep it
			_set_perm_alarm();    # set permanent for next run
		}
	}
	elsif ( test_perm_alarm() ) {

		# no alarm in current module : end of alarm

		# send a mail and log
		my $text = "alarm recovery on module $module_name";
		print "$text\n";
		journaliser( get_aquisition_journal(), $text );

		# clear permanent file
		_reset_perm_alarm();

	}
	return;
}

############################################################
# retrieve text from alarm file in single line
sub get_alarm_msg($) {
	my $module_dir = shift @_ || $localdir;

	my $alarm_file = $module_dir . $alarm;

	my $msg = EMPTY();
	if ( open my $fh, '<', $alarm_file ) {
		my @tab = <$fh>;
		$msg .= join SPACE(), @tab;
		## no critic (RequireCheckedSyscalls,RequireCheckedClose)
		close $fh;
		## use critic
	}
	return $msg;
}
############################################################
sub clear_alarm($) {
	my $module_dir = shift @_ || $localdir;

	if ( test_perm_alarm($module_dir) ) {
		_reset_perm_alarm($module_dir);
	}
	else {
		debug("no alarm on $module_dir");
	}
	return;
}
############################################################
1;
