
###############################################################################
#    Stats Webmin Module
#    Copyright (C) 2002 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this lib is used by all aquisition scripts (launched by cron)
# it depens upon the general lib sysstats-gen-lib.pl
###############################################################################

use strict;
use warnings;
use Cwd;
use File::Basename;
use English qw(-no_match_vars);

# is set in path by the caller with a use lib
## no critic (RequireBarewordIncludes)
require 'sysstats-gen-lib.pl';
require 'sysstats-alarm-lib.pl';
require 'sysstats-action-lib.pl';
require 'sysstats-rrd-lib.pl';
## use critic

#####################################################################################
# is_debug_mode
sub is_debug_mode() {
	return exists $ENV{DEBUG_MODE};
}
#####################################################################################
# used for debug mode
sub debug(@) {
	print 'debug ' . localtime() . q{ } . get_caller() . " : @_\n"
	  if ( is_debug_mode() );
	return;
}
#####################################################################################
# used for debug mode
sub info(@) {
	print 'info ' . get_caller() . " : @_\n";
	return;
}
#####################################################################################
sub get_module() {
	return ( exists $ENV{'module_name'} )
	  ? "on module $ENV{'module_name'} "
	  : EMPTY();
}
#####################################################################################
# used to inform on problems
sub warning(@) {
	set_cur_alarm(@_);    # alarm in current run
	if ( test_perm_alarm() ) {

		# avoid to send a new mail
		# get first line
		debug( 'SKIP ALARM ' . $_[0] );
	}
	else {
		my $text = 'WARNING ' . get_module() . get_caller() . " : @_";
		warn "$text\n";

		journaliser( get_aquisition_journal(), $text );
	}
	return;
}
###############################################################################
sub my_die(@) {
	set_cur_alarm(@_);    # alarm in current run
	if ( test_perm_alarm() ) {

		# avoid to send a new mail
		debug( 'SKIP ALARM ' . $_[0] );
	}
	else {
		warn 'DYING from ' . get_module() . get_caller() . " : @_\n";
	}
	exit;
}

#####################################################################################
# because with old version, unit was in line by minute (gauge)
# with derive, it is in line by seconds
# for compatibility, we transform the data : 60 * (data2 - data1)/60 = (60*data2 - 60*data1)/60
sub sec_2_min($) {
	my $nb = shift @_;
	$nb *= 60 if ( $nb ne 'U' );
	return $nb;
}

###############################################################################
# log
###############################################################################
# use var to pass arg from general config to modules
# used by test_log_size (below)
{
	my $max_log_size = 0;

	# store value
	sub set_max_log_size($) {
		my $val = shift @_;
		$max_log_size = $val;
		return;
	}

	# get value
	sub get_max_log_size() {
		return $max_log_size;
	}
}
###############################################################################
# test_log_size
# return 1 if ok (size is minus than max_log_size)
sub test_log_size($) {
	my $fic = shift @_;
	my $max = get_max_log_size();

	my $size = ( stat $fic )[7] || 0;

	# to test with configured max size
	if ( ( $max != 0 ) and ( $size > $max ) ) {
		warning("$fic size is $size, more than $max limit");
		return 0;
	}
	else {
		return 1;
	}
}
###############################################################################
# return true if is to be runned
# note runstop = 0 for run ; 1 for stop
sub is_param_runstop($) {
	my $runstop = $_[0];
	return ( $runstop == 0 );
}
###############################################################################
# chrono
###############################################################################
# start a "chrono"
# return time of begin
## no critic (ProhibitPackageVars)
our $chrono_start;

# stop a chrono
# and return time in second since the argument
our $chrono_stop;
## use critic

# implement 3 methods to chrono :
# Time::HiRes : is a non standard module
# syscall.ph : does not work on all plateforms
# time : is a very poor solution : precision in seconds
eval { require Time::HiRes };
if ($EVAL_ERROR) {
	debug('gen-lib : Time::HiRes not found');
	## no critic (RequireBarewordIncludes)
	eval { require 'syscall.ph' };
	## use critic
	if ($EVAL_ERROR) {
		debug('gen-lib : syscall.ph not found');
		$chrono_start = \&chrono_start_time;
		$chrono_stop  = \&chrono_stop_time;
	}
	else {
		$chrono_start = \&chrono_start_syscall;
		$chrono_stop  = \&chrono_stop_syscall;
	}
}
else {
	$chrono_start = \&chrono_start_hires;
	$chrono_stop  = \&chrono_stop_hires;
}
############################################################
# Time::HiRes methode
sub chrono_start_hires() {
	return Time::HiRes::gettimeofday();
}

sub chrono_stop_hires($) {
	my $debut = shift @_;
	return Time::HiRes::gettimeofday() - $debut;
}
############################################################
# syscall method (perl cookbook 3.9)
my $TIMEVAL_T = 'LL';

sub chrono_start_syscall () {
	my $debut = pack $TIMEVAL_T, ();

	# get date/time in $debut
	## no critic (ProhibitAmpersandSigils)
	syscall( &SYS_gettimeofday, $debut, 0 ) != -1
	  || warning("gettimeofday : $ERRNO");
	## use critic

	return $debut;
}

sub chrono_stop_syscall ($) {
	my $debut = shift @_;
	my $fin = pack $TIMEVAL_T, ();

	# get date/time in $fin
	## no critic (ProhibitAmpersandSigils)
	syscall( &SYS_gettimeofday, $fin, 0 ) != -1
	  || warning("gettimeofday : $ERRNO");
	## use critic

	my @debut = unpack $TIMEVAL_T, $debut;
	my @fin   = unpack $TIMEVAL_T, $fin;

	# change from micro-second to seconds
	for ( $fin[1], $debut[1] ) {
		$_ /= 1_000_000;
	}

	my $delta = sprintf '%.4f',
	  ( $fin[0] + $fin[1] ) - ( $debut[0] + $debut[1] );

	return $delta;
}
############################################################
# time method
sub chrono_start_time () {
	return time;
}

sub chrono_stop_time($) {
	my $debut = shift @_;

	return time - $debut;
}
############################################################
# files
############################################################
# build file names to keep current position
# name is file . module . 'pos'
sub get_pos_name($$) {
	my $ficname     = shift @_;
	my $module_name = shift @_;

	my $dir_pos  = $ENV{'WEBMINSTAT_TEMP'};
	my $base_fic = basename($ficname);

	return "$dir_pos/$base_fic.$module_name.pos";
}
############################################################
# get previous position
sub get_pos($$) {
	my $ficname     = shift @_;
	my $module_name = shift @_;

	my $previous_pos = 0;
	my $whence       = 0;
	my $fh_pos;
	my $pos_name = get_pos_name( $ficname, $module_name );

	if ( !-f $pos_name ) {

		# first use
		# to avoid a first too big value, we do not set pos to 0
		# but at the end of file
		$previous_pos = 0;
		$whence       = 2;
	}
	else {
		my $size = ( stat $pos_name )[7];
		if ($size) {

			# ok
			## no critic (RequireBriefOpen)
			if ( open $fh_pos, '<', $pos_name ) {
				$previous_pos = <$fh_pos>;
				my_close( $fh_pos, $ficname );
				if ( !defined $previous_pos ) {
					$previous_pos = 0;
					warning(
						"$module_name pos file $pos_name : undefined pos value"
					);
				}
			}
			else {
				# problem
				$previous_pos = 0;
				$whence       = 2;
				warning(
					"$module_name : can not open pos file $pos_name : $ERRNO");
			}
		}
		else {
			# problem : empty file !
			$previous_pos = 0;
			$whence       = 2;
			warning("$module_name : empty pos file $pos_name");
		}
	}
	return ( $pos_name, $previous_pos, $whence );
}
############################################################
# open a file at last read position
# return file handle to file
sub open_fic($$) {
	my $ficname     = shift @_;
	my $module_name = shift @_;

	# first check if name is defined
	if ( !$ficname ) {
		warning("$module_name : undefined file name");
		return;
	}

	my ( $pos_name, $previous_pos, $whence ) =
	  get_pos( $ficname, $module_name );

	# open file at previous position
	################################
	## no critic (RequireBriefOpen)
	my $fh;
	if ( !open $fh, '<', $ficname ) {
		warning("$module_name : can not open $ficname : $ERRNO");
		return;
	}

	# check size against pos
	my $size = ( stat $fh )[7];
	$previous_pos = 0 if ( $size < $previous_pos );

	debug("$pos_name previous_pos $previous_pos");

	# go to last pos
	seek $fh, $previous_pos, $whence;

	# we return the 2 value (file header and name)
	# as we need file header to read and parse the file
	# and we nees pos_name to close it with close_fic sub
	return ( $fh, $pos_name );
}
############################################################
# close a file and save position
sub close_fic($$;$) {
	my $fh       = shift @_;              # file descriptor (open file)
	my $pos_name = shift @_;              # pos file name
	my $filename = shift @_ || 'dummy';

	# get current position
	my $pos = tell $fh;
	my_close( $fh, $filename );

	# check
	if ( defined $pos ) {
		debug("$pos_name new_pos $pos");
	}
	else {
		# fix pos
		$pos = 0;
		warning("can not retrieve pos value from $filename");
	}

	# save pos
	my $fh_pos;
	## no critic (RequireBriefOpen)
	if ( open $fh_pos, '>', $pos_name ) {
		print {$fh_pos} $pos;
		my_close( $fh_pos, $pos_name );
	}
	else {
		warning("can not write pos in $pos_name : $ERRNO");
	}

	return;
}
###############################################################################
# read new lines from a file
# to have only one read even if several calls on same file
my %cache_fic;

sub read_new_lines($$) {
	my $filename    = shift @_;
	my $module_name = shift @_;

	if ( !exists $cache_fic{$filename} ) {
		my @tab = ();

		# populate cache
		my ( $fh, $pos_name ) = open_fic( $filename, $module_name );
		if ($fh) {
			@tab = <$fh>;
			close_fic( $fh, $pos_name, $filename );
		}
		$cache_fic{$filename} = \@tab;
	}
	return $cache_fic{$filename};
}
###############################################################################
# count the number of lines in a file
# rem : used by generic and internet module
sub count_files_lines($$) {
	my $fic         = shift @_;    # file name to open
	my $module_name = shift @_;    # module name (necessary for open_fic)

	my $total = 0;
	test_log_size($fic);

	my $r_tab = read_new_lines( $fic, $module_name );
	foreach ( @{$r_tab} ) {
		$total++;
	}

	return $total;
}
###############################################################################
# read_file(file, &assoc, [&order], [lowercase])
# Fill an associative array with name=value pairs from a file
# code from webmin :  web-lib.pl
sub read_file($$;$) {
	my $filename = shift @_;
	my $assoc    = shift @_;    # ref to hash to be filled
	my $order    = shift @_;    # ref to an array to be filled
	my $lcflag   = shift @_;

	my $fh_config;
	## no critic (RequireBriefOpen)
	if ( open $fh_config, '<', $filename ) {
		while (<$fh_config>) {
			chomp;

			# to skip commented lines
			my $hash = index $_, DIESE();

			# to skip lines without a "="
			my $eq = index $_, EQUAL();
			if ( ( $hash != 0 ) and ( $eq >= 0 ) ) {
				my $n = substr $_, 0, $eq;
				my $v = substr $_, $eq + 1;
				$assoc->{ $lcflag ? lc $n : $n } = $v;
				## no critic (ProhibitParensWithBuiltins)
				push( @{$order}, $n ) if ($order);
				## use critic
			}
		}
		my_close( $fh_config, $filename );

		# empty cache and fill it
		## no critic (ProhibitPackageVars)
		undef $main::read_file_cache;
		%{ $main::read_file_cache{$filename} } = %{$assoc};
		## use critic
		return 1;
	}
	else {
		warning( "can not open $filename (" . getcwd() . "): $ERRNO" );
		return 0;
	}
}
#####################################################################################
# read_file_cached(file, &assoc)
# Like read_file, but reads from a cache if the file has already been read
# code from webmin : web-lib.pl
sub read_file_cached($$) {
	my $filename = shift @_;
	my $assoc    = shift @_;

	my $ret;
	## no critic (ProhibitPackageVars)
	if ( defined $main::read_file_cache{$filename} ) {

		# add data from cache to %assoc
		%{$assoc} = ( %{$assoc}, %{ $main::read_file_cache{$filename} } );
		## use critic
		debug("read_file_cached cache found on $filename");
		$ret = 1;
	}
	else {

		# no cache, read data from file
		my %d;
		$ret = read_file( $filename, \%d );

		# and data to %assoc
		%{$assoc} = ( %{$assoc}, %d );
		debug("read_file_cached : read_file on $filename");
	}
	$ENV{'module_name'} = $assoc->{'name'};
	return $ret;
}
############################################################
# to be called at begin of each module aquisition code
sub init_module_run($$) {
	my $rh_config = shift @_;
	my $rh_codes  = shift @_;

	if ( !read_module_config($rh_config) ) {
		return;
	}
	my $module_name = $rh_config->{'name'};
	$ENV{'module_name'} = $module_name;
	debug( "run $module_name version " . $rh_config->{'version'} );
	%{$rh_codes} = action_load($rh_config);

	return $module_name;
}
############################################################
# main
############################################################
$ENV{'LC_NUMERIC'} = 'POSIX';

# set temporary directory and create it if necessary
my $WEBMINSTAT_TEMP_DIR = get_tempdir();
$ENV{'WEBMINSTAT_TEMP'} = $WEBMINSTAT_TEMP_DIR;
touch( get_aquisition_journal() );
############################################################
1;
