
###############################################################################
#    Webmin Sysstats
#    Copyright (C) 2013 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# a lib dedicated to clone operations
###############################################################################
# there are 2 problems with clones
# - module upgrade
#	if the src module change, the cloned code should change too
#	this can be done by symlinks on code, but not on meta-data
# - webminstats upgrade
# 	if webminstats is upgraded, /usr/libexec/webmin/sysstats is lost
# 	but not /etc/webmin/sysstats : we have to keep a track of all clones
# 	and "re-clones" then in post-intall
###############################################################################

use strict;
use warnings;
use English qw(-no_match_vars);
use File::Copy;
## no critic (ProhibitPackageVars)
use vars
  qw( %gconfig %config %text $module_config_directory $cb $tb %in %access);
## use critic

my $EMPTY = EMPTY();
my $SLASH = SLASH();

my $clone_file = get_tempdir() . 'clones';

my %cache_clone;    # cache
###############################################################################
# read register file
sub _read_clones() {
	if ( !%cache_clone ) {
		if ( -f $clone_file ) {
			read_file( $clone_file, \%cache_clone );
		}
		else {
			%cache_clone = ();
		}
	}
	return \%cache_clone;
}
###############################################################################
# write register file
sub _write_clones($) {
	my $r_h = shift @_;

	write_file_hash( $clone_file, $r_h );
	%cache_clone = %{$r_h};
	return;
}
###############################################################################
# register a module clone
sub clone_reg($$) {
	my $src   = shift @_;
	my $clone = shift @_;

	my $r_h = _read_clones();
	$r_h->{$clone} = $src;
	_write_clones($r_h);
	return;
}
###############################################################################
# remove a module clone
# to be called in delete_mod.cgi
sub clone_unreg($) {
	my $mod = shift @_;

	if ( is_clone($mod) ) {
		my $r_h = _read_clones();
		delete $r_h->{$mod};
		_write_clones($r_h);
	}
	return;
}
###############################################################################
# return true (the src) if the given module name is a clone
# else return false
sub is_clone($) {
	my $mod = shift @_;

	my $r_h = _read_clones();
	return ( exists $r_h->{$mod} ) ? $r_h->{$mod} : undef;
}
###############################################################################
# return true if the given module is cloned
# rem : is used in delete_mod.cgi
sub is_cloned($) {
	my $mod = shift @_;

	my $r_h = _read_clones();
	foreach my $m ( keys %{$r_h} ) {
		return $m if ( $r_h->{$m} eq $mod );
	}
	return 0;
}
###############################################################################
# clone a module
sub clone_mod($$$) {
	my $src     = shift @_;    # source module
	my $clone   = shift @_;    # clone name
	my $flag_re = shift @_;    #  0 if first clone
	                           #  1 if re-clone
	                           #  2 if clone with config/data

	# save current directory
	my $pwd = getcwd();

	my $sysstats_dir = sysstat_dir();
	chdir "$sysstats_dir/modules";

	# check for src
	if ( !-d $src ) {
		warning("source module $src does not exists : can not clone");
		return;
	}

	# check if the dest name already exists
	# directory for code
	if ( -e $clone ) {
		warning("clone dest $clone already exists : can not clone");
		return;
	}

	# config directory
	my $new_config = "$module_config_directory/modules/$clone";
	if ( $flag_re == 1 ) {

		# new_config should exists
		if ( !-d $new_config ) {
			warning("clone dest $new_config should exists");
			mkdir $new_config;
		}
	}
	elsif ( -e $new_config ) {
		warning("clone dest $new_config already exists : can not clone");
		return;
	}
	else {

		# create config dir
		mkdir $new_config;
	}

	# ok, let's clone : /usr/libexec/webmin/sysstats
	info("create $sysstats_dir/modules/$clone");
	mkdir $clone;
	chdir $clone;

	# make symlinks
	my @list = glob "../$src/*";
	foreach my $fic (@list) {

		# new name
		my $name = basename($fic);
		$name =~ s/$src/$clone/;
		if ( !-e $name ) {
			if ( $name eq 'module.info' ) {
				my_copy( $fic, $name );
				info("copy $fic to $name");
			}
			elsif ( $name eq 'lang' ) {
				my_recursive_copy( $fic, $name );
				info("recursive copy $fic to $name");
			}
			else {

				# symlink
				my_symlink( $fic, $name );
				info("$fic link to $name");
			}
		}
	}

	# change module_name to $clone in module.info
	chdir $sysstats_dir;
	my %h;
	my $minfo_file = "modules/$clone/module.info";
	read_file( $minfo_file, \%h );
	$h{'module_orig'} = $h{'module_name'};    # save origin
	$h{'module_name'} = $clone;
	write_file_hash( $minfo_file, \%h );

	# change module name in lang (module_title_index)
	my @langs = glob "modules/$clone/lang/*";
	foreach my $lang (@langs) {
		%h = ();
		read_file( $lang, \%h );
		$h{'module_title_index'} = $clone;
		write_file_hash( $lang, \%h );
	}

	# install config
	if ( $flag_re == 0 ) {
		my %defaults;
		my %minfo = get_sysstats_module_info($clone);
		try_install_module( $clone, \%minfo, \%defaults );

		# register clone module
		clone_reg( $src, $clone );
	}
	elsif ( $flag_re == 1 ) {

		# nothing to do
		info("re-clone $src to $clone");
	}
	elsif ( $flag_re == 2 ) {

		# copy config
		my_copy(
			"$module_config_directory/modules/$src/config",
			"$module_config_directory/modules/$clone/config"
		);

		# copy data
		my @list_rrd = glob "$module_config_directory/modules/$src/*.rrd";
		foreach my $rrd (@list_rrd) {

			# new name
			my $rrd_new = $rrd;
			$rrd_new =~ s/$src/$clone/;
			my_copy( $rrd, $rrd_new );
		}

		# aquisition script
		my $plsrc = "$sysstats_dir/modules/$clone/$clone.pl";
		my $pldst = "$module_config_directory/modules/$clone/$clone.pl";
		my_symlink( $plsrc, $pldst );
		info("link $plsrc to $pldst");

		# register clone module
		clone_reg( $src, $clone );
	}

	chdir $pwd;
	return;
}
###############################################################################
# after a webminstats update, all clone are lost, but not data
# we have to re-clone registered modules
# to be called by postinstall.pl
sub re_clone() {

	my $r_h = _read_clones();
	foreach my $clone ( keys %{$r_h} ) {
		clone_mod( $r_h->{$clone}, $clone, 1 );
	}

	return;
}
###############################################################################
1;
