
###############################################################################
#    Webmin Sysstats
#    Copyright (C) 2004 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# a lib dedicated to cron operations (start and stop aquisition)
# we use the modern /etc/cron.d/ directory if it exists
# else we try to use the old "crontab" system with the cron webmin module
###############################################################################

use strict;
use warnings;
use English qw(-no_match_vars);
use File::Copy;

my $EMPTY = EMPTY();
my $SLASH = SLASH();

## no critic (ProhibitPackageVars)
use vars qw( $cron_dir %config );
use vars qw( $module_name );

## use critic

###############################################################################
# to avoid a global variable to share with others lib
sub cron_file() {

	#my $cron_file = 'webminstats';
	my $cron_file = $module_name;
	return $cron_file;
}
###############################################################################
# test for "new" cron (easier to use)
sub is_new_cron() {
	return ( -d '/etc/cron.d' );
}
###############################################################################
# detect is webminstats cron is installed
sub is_webmin_sysstats_running() {
	debug_sub_begin();

	if ( is_new_cron() ) {
		my $cron = $cron_dir . cron_file();
		return ( -f $cron );
	}
	else {
		return is_webmin_sysstats_running_old();
	}
}
###############################################################################
# for old cron system (crontab command)
sub is_webmin_sysstats_running_old() {
	debug_sub_begin();

	my $cmd = aquisition_script();

	wrapper( 'foreign_require', 'cron', 'cron-lib.pl' );

	unset_warnings();
	## no critic (ProhibitAmpersandSigils)
	my @jobs = &foreign_call( 'cron', 'list_cron_jobs' );
	## use critic
	set_warnings();

	my $rep = 0;
	foreach (@jobs) {
		if ( ( $_->{'command'} eq $cmd ) and ( $_->{'active'} ) ) {
			$rep++;
		}
	}
	debug_sub_end();

	# return the number of cron jobs
	return $rep;
}

###############################################################################
# stop aquisition (remove cron)
sub stop_sysstats() {
	debug_sub_begin();

	if ( is_new_cron() ) {
		unlink $cron_dir . cron_file();
	}
	else {
		delete_cron_task( aquisition_script() );
	}

	my_webmin_log( 'stop', 'cron', 'aquisition' );

	debug_sub_end();
	return;
}

###############################################################################
# build a new cron file depending about module name (for webmin clones)
sub build_cron_file() {
	my $cronfile = cron_file();
	my $script   = aquisition_script();

	my $mailto = ( $config{'mailto'} ) ? $config{'mailto'} : 'root';

	if ( open my $fh, '>', $cronfile ) {
		print {$fh} "MAILTO=$mailto\n";
		print {$fh} "* * * * *    root	$script\n";
		close $fh or warning("can not close $cronfile : $ERRNO");
	}
	else {
		warning("can not create $cronfile : $ERRNO");
	}
	return;
}
###############################################################################
# start aquisition (add cron)
sub start_sysstats() {
	debug_sub_begin();
	if ( is_new_cron() ) {

		my $sysstat_dir = sysstat_dir();

		build_cron_file();

		# copy file
		copy( $sysstat_dir . cron_file(), $cron_dir . cron_file() );
		chmod oct 644, $cron_dir . cron_file();
	}
	else {

		# set a cron task every minute
		my $STAR = STAR();
		my $fic  = aquisition_script();
		create_cron_task( 'root', $fic, $EMPTY, $STAR, $STAR, $STAR, $STAR,
			$STAR );
	}

	my_webmin_log( 'start', 'cron', 'aquisition' );
	debug_sub_end();
	return;
}

###############################################################################
#		old cron section
# it works by using another webmin module : cron
# to be used with "old" unix systems without /etc/cron.d/ directory
# this code does not use clean sub call with prototype
###############################################################################
# create a cron task
# no critic (ProhibitManyArgs)
sub create_cron_task($$$$$$$$) {
	debug_sub_begin();

	## no critic (ProhibitAmpersandSigils)
	my $job = { 'type' => 0 };

	my $user  = $_[0];
	my $cmd   = $_[1];
	my $input = $_[2];
	$job->{'mins'}     = $_[3];
	$job->{'hours'}    = $_[4];
	$job->{'days'}     = $_[5];
	$job->{'months'}   = $_[6];
	$job->{'weekdays'} = $_[7];

	# Check and parse inputs
	if ( !$user ) {
		text_error('cron_save_euser');
	}
	elsif ( !defined getpwnam $user ) {
		text_error( 'cron_save_euser2', $user );
	}

	# Check if this user is allowed to execute cron jobs
	wrapper( 'foreign_require', 'cron', 'cron-lib.pl' );
	my %cron_config = &foreign_config('cron');
	my $err;

	## no critic (ProhibitCascadingIfElse)
	if ( -r $cron_config{'cron_allow_file'} ) {
		if ( &indexof( $user, &foreign_call( 'cron', 'list_allowed' ) ) < 0 ) {
			$err = 1;
		}
	}
	elsif ( -r $cron_config{'cron_deny_file'} ) {
		if ( &indexof( $user, &foreign_call( 'cron', 'list_denied' ) ) >= 0 ) {
			$err = 1;
		}
	}
	elsif ( $cron_config{'cron_deny_all'} == 0 ) {
		$err = 1;
	}
	elsif ( $cron_config{'cron_deny_all'} == 1 ) {
		if ( $user ne 'root' ) {
			$err = 1;
		}
	}

	if ($err) {
		error( text( 'cron_save_eallow', $user ) );
	}

	# Check module access control
	my %cron_access = get_module_acl( undef, 'cron' );
	&foreign_call( 'cron', 'can_edit_user', \%cron_access, $user )
	  || error( text( 'cron_save_ecannot', $user ) );

	unset_warnings();
	my @jobs = &foreign_call( 'cron', 'list_cron_jobs' );
	set_warnings();

	my @files = &unique( ( map { $_->{'file'} } @jobs ),
		"$cron_config{'cron_dir'}/$user}" );

	foreach my $f (@files) {
		wrapper( 'lock_file', $f );
	}

	$input =~ s/\r//g;
	$input =~ s/%/\\%/g;
	$cmd =~ s/%/\\%/g;
	$job->{'active'}  = 1;
	$job->{'command'} = $cmd;

	if ( $input =~ /\S/ ) {
		my @inlines = split /\n/, $input;
		my $PERCENT = PCENT();
		$job->{'command'} .= $PERCENT . join $PERCENT, @inlines;
	}

	$job->{'user'} = $user;

	&foreign_call( 'cron', 'create_cron_job', $job );

	foreach my $f (@files) {
		&unlock_file($f);
	}
	## use critic

	debug_sub_end();
	return;
}

###############################################################################
# delete a cron task
# is only used on very old systems
# it use another webmin module : cron
sub delete_cron_task($) {
	my $cmd = $_[0];

	debug_sub_begin();

	## no critic (ProhibitAmpersandSigils)
	wrapper( 'foreign_require', 'cron', 'cron-lib.pl' );
	my %cron_config = &foreign_config('cron');

	my %cron_access = get_module_acl( undef, 'cron' );

	unset_warnings();
	my @jobs = &foreign_call( 'cron', 'list_cron_jobs' );
	set_warnings();

	foreach my $job ( reverse @jobs ) {
		if ( ( $job->{'command'} eq $cmd ) and ( $job->{'active'} ) ) {

			# delete all jobs
			wrapper( 'lock_file', $job->{'file'} );
			&foreign_call( 'cron', 'can_edit_user', \%cron_access,
				$job->{'user'} )
			  || text_error('cron_del_ecannot');
			&foreign_call( 'cron', 'delete_cron_job', $job );
			&unlock_file( $job->{'file'} );
		}
	}
	## use critic

	debug_sub_end();
	return;
}
###############################################################################
1;
