
###############################################################################
#
#    Stats Webmin Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################
# this is a very general lib, with sub to be shared by all
###############################################################################
use strict;
use warnings;

use English qw(-no_match_vars);
use File::Copy;
use POSIX qw( strftime );

## no critic (RequireBarewordIncludes)
require 'sysstats-constants-lib.pl';
require 'sysstats-alarm-lib.pl';
## use critic

## no warning 'RequireLocalizedPunctuationVars'
###############################################################################
# set perl warnings on
sub set_warnings() {

	#use warnings 'all';
	$WARNING = 1;
	return;
}
###############################################################################
# set perl warnings off
sub unset_warnings() {

	#no warnings 'all';
	$WARNING = 0;
	return;
}
## use critic
##############################################################################
{

	# static var to store last error messages
	my $error = EMPTY();

	# store a message
	sub set_error($) {
		$error = shift @_;
		return;
	}

	# return and reset the message
	sub get_error() {
		my $msg = $error;

		# reset error
		$error = EMPTY();
		return $msg;
	}
}
##############################################################################
# 			new printing sub
##############################################################################
# return infos of subroutine caller
sub get_caller(;$) {
	my $level = shift @_ || 1;
	my (
		$package, $filename, $line, undef, undef,
		undef,    undef,     undef, undef, undef
	) = caller $level++;
	my (
		undef, undef, undef, $subroutine, undef,
		undef, undef, undef, undef,       undef
	) = caller $level;
	$subroutine = $subroutine || 'main';
	return "${subroutine}($filename $line) ";
}
###############################################################################
# get operating system name
# cf http://perldoc.perl.org/perlport.html#PLATFORMS
sub get_os() {
	return $OSNAME;
}
###############################################################################
# is operating system linux ?
sub is_linux() {
	return ( get_os() =~ m/linux/i );
}
###############################################################################
# is operating system bsd
sub is_bsd() {
	return ( get_os() =~ m/bsd/i );
}
###############################################################################
# is operating system macos
sub is_macos() {
	return ( get_os() =~ m/darwin/i );
}
###############################################################################
#              arg control section
###############################################################################
# ex : 134
sub is_unsigned_int($) {
	my $value = shift @_;

	return ( $value =~ m/^\d+$/ );
}
###############################################################################
# ex : -134
sub is_signed_int($) {
	my $value = shift @_;

	return ( $value =~ m/^[+-]?\d+$/ );
}
###############################################################################
# test for an unsigned value ( ex 0.99)
sub is_unsigned_real($) {
	my $value = shift @_;

	return ( $value =~ m/^\d+[.]?\d*$/ );
}
###############################################################################
# test for a signed value ( ex -0.99)
sub is_signed_real($) {
	my $value = shift @_;

	return ( $value =~ m/^[+-]?\d+[.]?\d*$/ );
}
###############################################################################
# open and close a file : so empty the file
sub empty_file($) {
	my $name = shift @_;

	if ( -f $name ) {

		# save file
		copy( $name, $name . '.old' ) or warning("can not copy $name : $ERRNO");
	}

	my $fh_fic;
	## no critic (RequireBriefOpen)
	if ( open $fh_fic, '>', $name ) {
		my_close( $fh_fic, $name );
	}
	else {
		warning("can not open $name : $ERRNO");
	}
	return;
}
###############################################################################
sub write_file_txt($$) {
	my $name = shift @_;
	my $txt  = shift @_;

	my $fh_fic;
	## no critic (RequireBriefOpen)
	if ( open $fh_fic, '>', $name ) {
		print {$fh_fic} $txt;
		my_close( $fh_fic, $name );
	}
	else {
		warning("can not open $name : $ERRNO");
	}
	return;
}
###############################################################################
# same as the touch command
# will create the file or change it's modification time
sub touch($) {
	my $name = shift @_;

	if ( -f $name ) {

		# undef only works Since perl 5.7.2
		my $mtime = time;
		utime $mtime, $mtime, $name;
	}
	else {
		empty_file($name);
	}
	return;
}
###############################################################################
# we want to keep config order, which is a mixed of alpha and alpha+numeric
# (  'uid', 'custom', 'vol1', 'vol5', 'vol10' ... )
# cf perldoc sort
sub tri_config(@) {

	no warnings;
	return map { $_->[0] }
	  sort { $a->[1] <=> $b->[1] || $a->[2] cmp $b->[2] }
	  map { [ $_, /(\d+)/, $_ ] } @_;
}
###############################################################################
# return the sorted keys in string separated by a space char
sub sort_keys($) {
	my $r_h = shift @_;

	return join q{ }, sort keys %{$r_h};
}
###############################################################################
# write a sorted hash
# because it is much easier to read by a human
# it is inspired from write_file sub (from web-lib-funcs.pl : webmin)
sub write_file_hash($$) {

	my $filename = shift @_;    # file name to write to
	my $r_config = shift @_;    # reference to the hash structure

	my $EMPTY = EMPTY();

	my $fh;
	## no critic (RequireBriefOpen)
	if ( open $fh, '>', $filename ) {
		foreach my $k ( tri_config( keys %{$r_config} ) ) {

			# set a default value to empty
			my $value = ( defined $r_config->{$k} ) ? $r_config->{$k} : $EMPTY;
			print {$fh} "$k=$value\n";
		}
		my_close( $fh, $filename );

		# update cache : see also  read_file and read_file_cached
		## no critic (ProhibitPackageVars)
		if ( defined $main::read_file_cache{$filename} ) {
			%{ $main::read_file_cache{$filename} } = %{$r_config};
		}
		## use critic
	}
	else {

		# error
		warning("can not write to $filename : $ERRNO");
	}

	return;
}
###############################################################################
# read a list or items in a config file
sub read_list($) {
	my $filename = shift @_;

	my %hash;
	my $r_tab = read_full_file($filename);
	foreach my $ligne ( @{$r_tab} ) {
		chomp $ligne;
		next if ( $ligne =~ m/^#/ );       # comments
		next if ( $ligne =~ m/^\s*$/ );    # empty
		$hash{$ligne} = 1;
	}

	return %hash;
}
###############################################################################
sub read_awk($) {
	my $filename = shift @_;

	my %hash;
	my $fh;
	## no critic (RequireBriefOpen)
	if ( open $fh, '<', $filename ) {
		while ( my $ligne = <$fh> ) {
			chomp $ligne;
			next if ( $ligne =~ m/^#/ );       # comments
			next if ( $ligne =~ m/^\s*$/ );    # empty
			my ( $key, $value ) = split_awk($ligne);

			$hash{$key} = $value;
		}
		my_close( $fh, $filename );
	}
	else {
		warning("can not open $filename : $ERRNO");
	}

	return %hash;
}
###############################################################################
# to return the content a directory without the . and .. special files
sub readdir_content($) {
	my $directory = shift @_;

	my $fh_dir;
	if ( opendir $fh_dir, $directory ) {

		my @fic = grep !/^[.]{1,2}$/, readdir $fh_dir;
		closedir $fh_dir
		  or warning("can not close directory $directory : $ERRNO");
		return @fic;
	}
	else {
		warning("can not open directory $directory : $ERRNO");
		return;
	}
}
###############################################################################
#exchange value of 2 parameters
sub exchange($$) {
	my $r_val1 = shift @_;
	my $r_val2 = shift @_;

	# cf perl cookbook
	( ${$r_val1}, ${$r_val2} ) = ( ${$r_val2}, ${$r_val1} );

	# old with intermediate val
	#my $tmp = $$r_val1;
	#$$r_val1 = $$r_val2;
	#$$r_val2 = $tmp;
	return;
}
###############################################################################
# common path with system directories
sub common_path() {
	return ':/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/sbin/:/usr/local/bin/';
}
###############################################################################
# cf http://perltricks.com/article/115/2014/9/26/Shellshock-and-Perl
sub clean_env() {

	# shellshock
	foreach ( keys %ENV ) {
		delete $ENV{$_} if ( ( $ENV{$_} ) and ( $ENV{$_} =~ /};./ ) );
	}

	# perlsec
	delete @ENV{ 'IFS', 'CDPATH', 'ENV', 'BASH_ENV' };

	$ENV{'PATH'} .= common_path();
	$ENV{'SHELL'} = '/bin/sh';
	return;
}
###############################################################################
# returns the full path of the named program, or undef.
# test for an external program (lsof, firewall, custom)
sub search_command($) {
	my $prog = shift @_;

	debug("search_command : $prog");
	if ( $prog =~ m{^/} ) {

		# prog already contains an absolute path
		return ( -x $prog ) ? $prog : undef;
	}

 # we will search in current PATH but we force some admin directories
 # $ENV{'PATH'} .= common_path(); # already done in sysstats-lib and sysstats.pl
	foreach ( split /:/, $ENV{'PATH'} ) {
		if ( -x "$_/$prog" ) {
			debug("search_command : found $_/$prog");
			return "$_/$prog";
		}
	}
	return;
}
##########################################################
# check if a command exists and can executed
# return the full path to the command if ok
# else return undef if ko
sub test_command($) {
	my $command = shift @_;

	debug("test_command : $command");

	# if argument are set in command line
	my ( $cmd, @args ) = split_awk($command);

	my $ret_code;
	if ( !defined $cmd ) {
		$ret_code = undef;
		set_error('unconfigured command name');
	}
	else {

		# add full path to command
		if ( $cmd !~ m{^/} ) {
			$cmd = search_command($cmd);
		}

		# test
		if ( -x $cmd ) {
			$ret_code = $cmd;
			$ret_code .= " @args" if (@args);
		}
		else {
			$ret_code = undef;
			set_error("$cmd command can not be executed");
		}
	}
	return $ret_code;
}
##########################################################
# check if a file exists
# return true if ok
# else false
sub test_file($) {
	my $file = shift @_;

	my $ret_code;
	if ( ( !defined $file ) or ( $file eq EMPTY() ) ) {
		$ret_code = 0;
		set_error('unconfigured file');
	}
	elsif ( -r $file ) {
		$ret_code = 1;
	}
	else {
		$ret_code = 0;
		set_error("can not read $file");
	}

	return $ret_code;
}
##########################################################
# generic function to get file matching a pattern
sub get_file_names($$) {
	my $directory = shift @_;
	my $pattern   = shift @_;

	return glob $directory . SLASH() . $pattern;
}
##########################################################
# convert a module directory (generic-0.8.0) to a module name (generic)
#sub remove_version($) {
#	my $module = shift @_;    # for exemple : generic-0.8.0
#
#	# just remove what is after the "-"
#	$module =~ s/-\d.*//;
#	return $module;
#}
##########################################################
# a special split call
sub split_awk($;$) {
	my $line  = $_[0];
	my $limit = $_[1];

	## no critic (ProhibitEmptyQuotes)
	if ( !defined $line ) {
		return;
	}
	elsif ( defined $limit ) {
		return split ' ', $line, $limit;
	}
	else {
		return split ' ', $line;
	}
}
##########################################################
# just a wrapper to call warning on problems
sub my_rename($$;$) {
	my $src  = shift @_;
	my $dest = shift @_;
	my $text = shift @_ || EMPTY();    # optionnal text

	if ( !rename $src, $dest ) {
		warning("$text pb to rename $src to $dest : $ERRNO");
	}
	else {
		debug("$text my_rename $src to $dest");
	}
	return;
}
##########################################################
# just a wrapper to call warning on problems
sub my_copy($$;$) {
	my $src  = shift @_;
	my $dest = shift @_;
	my $text = shift @_ || EMPTY();    # optionnal text

	if ( !copy( $src, $dest ) ) {
		warning("$text pb to copy $src to $dest : $ERRNO");
	}
	else {
		debug("$text my_copy $src to $dest");
	}
	return;
}
##########################################################
sub my_recursive_copy($$;$) {
	my $src  = shift @_;
	my $dest = shift @_;
	my $text = shift @_ || EMPTY();    # optionnal text

	if ( -d $src ) {
		if ( -e $dest ) {

			# protect againt copy on existing data
			warning(
				"my_recursive_copy avoid copy of $src to already existing $dest"
			);
		}
		else {
			system '/bin/cp', '-a', $src, $dest;
			debug("$text my_recursive_copy $src to $dest");
		}
	}
	else {
		my_copy( $src, $dest, $text );
	}
	return;
}
##########################################################
# just a wrapper to call warning on problems
sub my_symlink($$) {
	my $src  = shift @_;
	my $dest = shift @_;
	my $text = shift @_ || EMPTY();    # optionnal text

	if ( !symlink $src, $dest ) {
		warning("$text pb to symlink $src to $dest : $ERRNO");
	}
	else {
		debug("$text my_symlink $src to $dest");
	}
	return;
}
##########################################################
# wrapper on unlink function
sub my_unlink($;$) {
	my $fic = shift @_;
	my $text = shift @_ || EMPTY();    # optionnal text

	if ( -f $fic ) {
		if ( !unlink $fic ) {
			warning("$text can not remove $fic : $ERRNO");
		}
		else {
			debug("$text my_unlink $fic");
		}
	}
	else {
		debug("$text can not remove $fic : not exists");
	}

	return;
}
##########################################################
# close a file handle and display errno message if exists
sub my_close($$) {
	my $fh   = shift @_;    # file handle
	my $file = shift @_;    # file name for information

	if ( !close $fh ) {
		if ($ERRNO) {
			warning("can not close $file : $ERRNO");
		}
		else {
			warning("can not close $file");
		}
	}
	else {
		debug("close $file");
	}
	return;
}
##########################################################
sub my_close_pipe($$) {
	my $fh   = shift @_;    # file handle
	my $file = shift @_;    # file name for information

	## no critic (RequireCheckedSyscalls,RequireCheckedClose)
	close $fh;
	debug("close pipe $file");
	return;
}
##########################################################
# read a file into an array and return it
# ps : avoid use this for a big file !
sub read_full_file($) {
	my $filename = shift @_;

	my $fh;
	## no critic (RequireBriefOpen)
	if ( open $fh, '<', $filename ) {
		my @tab = <$fh>;
		my_close( $fh, $filename );
		return \@tab;
	}
	else {
		warning("can not open $filename : $ERRNO");
		return;
	}
}
#####################################################################################
# in debug mode stderr goes to output
# in normal mode goes to dark hole
# rem : it is shell dependant (posix)
sub stderr_debug() {

	#return ' 2>&1 ';
	return is_debug_mode() ? ' 2>&1 ' : ' 2>/dev/null ';
}
##############################################################################
# read a command output
# could be implemented in different manners
# - with an open command and |
# - with backsticks
# - with IO::Pipe module
# avoid using a buffer to store data
# loop in calling program
sub read_pipe_stream($) {
	my $cmd = shift @_;

	debug("read_pipe : $cmd");

	#eval { require IO::Pipe };
	#if ($EVAL_ERROR) {

	# sol1 : open
	$cmd .= SPACE() . stderr_debug() . PIPE();
	my $fh;
	## no critic (RequireBriefOpen, ProhibitTwoArgOpen)
	if ( open $fh, $cmd ) {
		return $fh;
	}
	else {
		warning("can not open piped command $cmd : $ERRNO");
		return;
	}

	#else {

	# sol2 : IO::Pipe
	# my $pipe = IO::Pipe->new();
	# $pipe->reader($cmd);
	# return $pipe;
	#}
}
##########################################################
# read a command output and return the array
sub read_pipe($) {
	my $cmd = shift @_;

	my $pipe = read_pipe_stream($cmd);
	my @tab  = <$pipe>;
	## no critic (RequireCheckedSyscalls,RequireCheckedClose)
	close $pipe;

	chomp @tab;

	# test for data
	if ( !scalar @tab ) {
		info("warning on $cmd : no data returned");
		return;
	}
	else {
		return \@tab;
	}
}
##############################################################################
# a date in human format
sub logdate() {

	my @date = localtime;
	return strftime( '%Y/%m/%d %H:%M:%S ', @date );
}
##############################################################################
# append in a log file
sub journaliser($$) {
	my $file = shift @_;
	my $text = shift @_;

	if ( open my $fh, '>>', $file ) {
		print {$fh} logdate() . $text . "\n";

		# do not use my_close here else will loop to write on file
		# in debug mode
		## no critic (RequireCheckedSyscalls,RequireCheckedClose)
		close $fh;
	}
	else {
		warning("can not write to log file $file : $ERRNO");
	}
	return;
}
##########################################################
# tell if the module version is a beta version
# (contains alphabetic char)
sub is_beta_version($) {
	my $version = shift @_;

	return ( $version !~ m/^[\d.]+$/ );
}
##########################################################
# tell if the module version is a stable version
sub is_stable_version($) {
	my $version = shift @_;

	return ( $version >= 1.0 );
}
##########################################################
# a global cache for 'df -P -l' command
my $cache_df;

sub _get_df(;$) {
	my $opt = shift @_ || q{};    # ex -i, by disk module for inodes

	# -P is a posix option for the format
	# -k : fixe les unites en Ko
	# -l : report only local disks
	my $cmd = "/bin/df -P -l -k $opt";
	my $res = read_pipe($cmd);

	# skip first line (comments)
	shift @{$res};
	return $res;
}

sub get_df(;$) {
	my $opt = shift @_ || q{};    # ex -i, by disk module for inodes

	if ($opt) {

		# do not upgrade cache
		return _get_df($opt);
	}
	elsif ( !$cache_df ) {

		# upgrade cache
		$cache_df = _get_df();
	}

	# return cache
	return $cache_df;
}

##########################################################
# search the list of hard disk devices : /dev/sda /dev/sdb
# is used by diskio/hddtemp/smart modules
sub list_disk() {

	my @tab_devices;

	if ( is_linux() ) {

		# use sfdisk (linux)
		# /dev/sda: 732574584
		# /dev/sdb: 732574584
		# /dev/sdc:   1976320
		# total : 1467125488 blocs
		my $pipe = read_pipe_stream('sfdisk -s -q');
		while (<$pipe>) {
			if (m{^(/dev/.*):}) {
				my $device = $1;
				next if ( $device =~ m{/dev/ram} );
				next if ( $device =~ m{/dev/mapper} );
				next if ( $device =~ m{/dev/loop} );
				push @tab_devices, $device;
			}
		}
		## no critic (RequireCheckedSyscalls,RequireCheckedClose)
		close $pipe;
	}
	elsif ( is_bsd() ) {

		# ad 	ATAPI (IDE) disk
		# da 	SCSI direct access disk
		# egrep '^(ad|ada|da)[0-9]: ' /var/run/dmesg.boot
		# ad0: 152627MB at ata0-master UDMA33
		# ad1: 38166MB at ata0-slave UDMA33
		my %h;
		my $r_tab = read_full_file('/var/run/dmesg.boot');
		foreach my $ligne ( @{$r_tab} ) {
			if ( $ligne =~ m/^((ad|ada|da)\d): / ) {
				$h{$1} = 1;
			}
		}
		@tab_devices = keys %h;
	}
	else {
		warning( 'unsupported os in list_disk ' . get_os() );
	}
	my @sorted_devices = sort @tab_devices;
	return @sorted_devices;
}
##########################################################
# return list of disk partitions : /dev/sda1 /dev/sda2
# used in diskio module
sub list_partitions() {
	my @tab_devices;

	if ( is_linux() ) {

# use sfdisk (linux)
# Périph Amor Début     Fin   #cyls    #blocs    Id  Système
# /dev/sda1   *      0+   2548    2549-  20474811   83  Linux
# /dev/sda2       2549   47448   44900  360659250    5  Étendue
# /dev/sda3          0       -       0          0    0  Vide
# /dev/sda4          0       -       0          0    0  Vide
# /dev/sda5       2549+   3331     783-   6289416   82  partition d'échange Linux / Solaris
		my $pipe = read_pipe_stream('sfdisk -l -q');
		while (<$pipe>) {
			if (m{^(/dev/\S+)}) {
				my $device = $1;
				push @tab_devices, $device;
			}
		}
		## no critic (RequireCheckedSyscalls,RequireCheckedClose)
		close $pipe;
	}
	elsif ( is_bsd() ) {
		my $r_tab = get_df();
		foreach my $ligne ( @{$r_tab} ) {
			chomp $ligne;
			my ( $device, undef, undef, undef, undef, undef ) =
			  split_awk($ligne);
			next unless ( $device =~ m{^/dev/} );    # avoid rootfs
			push @tab_devices, $device;
		}
	}
	else {
		warning( 'unsupported os in list_partitions ' . get_os() );
	}
	my @sorted_devices = sort @tab_devices;
	return @sorted_devices;
}
##########################################################
# return list of mounted file systems
sub list_filesys() {

	my @tab_mount;
	my $r_tab = get_df();
	foreach my $ligne ( @{$r_tab} ) {
		chomp $ligne;
		my ( $device, undef, undef, undef, undef, $mount ) = split_awk($ligne);
		if ( $device =~ m{^/dev/} ) {
			push @tab_mount, $mount;
		}
	}
	return @tab_mount;
}
##########################################################
# given a mount point, return the device
# ex /home => /dev/sda9
sub fs2device($) {
	my $mnt = shift @_;    # ex : /home

	my $r_tab = get_df();
	foreach my $ligne ( @{$r_tab} ) {
		chomp $ligne;
		my ( $device, undef, undef, undef, undef, $mount ) = split_awk($ligne);
		next unless ( $device =~ m{^/dev/} );    # avoid rootfs
		if ( $mount eq $mnt ) {
			return $device;
		}
	}
	return;
}
##########################################################
# given a device, return the mount point
# ex /dev/sda9 => /home
sub device2fs($) {
	my $part = shift @_;    # ex : /dev/sda9

	my $r_tab = get_df();
	foreach my $ligne ( @{$r_tab} ) {
		chomp $ligne;
		my ( $device, undef, undef, undef, undef, $mount ) = split_awk($ligne);
		next unless ( $device =~ m{^/dev/} );    # avoid rootfs
		if ( $device eq $part ) {
			return $mount;
		}
	}
	return;
}
##########################################################
# change directory to parent directory (return from module dir)
sub chparent() {
	## no critic (ProhibitNoisyQuotes)
	chdir '..';
	## use critic
	return;
}
##########################################################
# return the first find file on disk from a list
sub guess_generic($) {
	my $r_tab = shift @_;    # list of possible files to check

	my $log = EMPTY();
	foreach my $t ( @{$r_tab} ) {
		if ( -f $t ) {
			$log = $t;
			last;
		}
	}

	return $log;
}
##########################################################
# compute pourcentage
sub pourcent($$) {
	my $val   = shift @_;
	my $total = shift @_;

	if ( $total == 0 ) {

		# avoid division by 0
		return 'U';
	}
	else {
		my $res = $val * 100 / $total;
		if ( $res < 102 ) {
			return sprintf '%.2f', $res;
		}
		else {
			debug("pourcent $res > 100");
			return 100;
		}
	}
}
##########################################################
# Find the number of installed processors in this machine
# linux-dependent function
sub find_num_cpus () {

	# all computer have ay least one processor
	my $nb_proc = 1;    # default

	if ( is_linux() ) {

		# sous linux
		my $info = '/proc/cpuinfo';

		my $r_tab = read_full_file($info);
		if ( scalar @{$r_tab} ) {

			my @cpu_list = grep /^processor/, @{$r_tab};

			# get a least one processor (powerpc fix)
			$nb_proc = scalar @cpu_list || 1;
		}
	}
	elsif ( is_bsd() ) {

		# sous bsd :
		## no critic (ProhibitBacktickOperators)
		$nb_proc = `/sbin/sysctl -n hw.ncpu`;
		chomp $nb_proc;
	}
	elsif ( is_macos() ) {

	 # http://osxdaily.com/2011/07/15/get-cpu-info-via-command-line-in-mac-os-x/
		## no critic (ProhibitBacktickOperators)
		my $out = `system_profiler | grep 'Number of Processors' `;
		chomp $out;
		( undef, $nb_proc ) = split /:/, $out;
	}

	return $nb_proc;
}
##########################################################
# is used in smart and ntp module to build unique name from
# 2 components (host + param)
my $sep_param1 = q{_};     # used in normal case
my $sep_param2 = q{__};    # used if the chain already contains the separator

sub encode_param($$) {
	my $param1 = shift @_;
	my $param2 = shift @_;

	if ( $param1 =~ m/$sep_param1/ ) {
		return $param1 . $sep_param2 . $param2;
	}
	else {
		return $param1 . $sep_param1 . $param2;
	}
}
##########################################################
sub decode_param($) {
	my $param = shift @_;

	if ( $param =~ m/$sep_param2/ ) {
		return split $sep_param2, $param, 2;
	}
	else {
		return split $sep_param1, $param, 2;
	}
}
###############################################################################
# ps code
###############################################################################
# ordered list of ps parameters
# full list is available in "man ps", in "STANDARD FORMAT SPECIFIERS" section
# from %cpu to wchan
# ps : command should be the last one because it can contains spaces
sub get_ps_list() {
	if ( is_bsd() ) {
		return (
			'flags',     # flags associated with the process
			'nice',      # nice value. This ranges from 19 (nicest) to -20
			'nivcsw',    # total involuntary context switches
			'nlwp',      #  number of threads in the process
			'nswap',     # total swaps in/out
			'nvcsw',     # total voluntary context switches
			'pcpu',      # cpu utilization as a percentage
			'pid',       # process ID number of the process
			'pmem'
			,  # ratio of the process's resident set size to the physical memory
			'ppid',       # parent process ID
			'rss',        # resident set size
			'rtprio',     # realtime priority
			'state',      # multi-character process state
			'time',       # cumulative CPU time
			'vsz',        # virtual memory size of the process
			'wchan',      # name of the kernel function sleeping
			'command',    # command with all its arguments as a string
		);
	}
	elsif ( is_linux() ) {

		# linux
		return (
			'euid',       # effective user ID
			'flags',      # flags associated with the process
			'nice',       # nice value. This ranges from 19 (nicest) to -20
			'nlwp',       #  number of threads in the process
			'pcpu',       # cpu utilization as a percentage
			'pid',        # process ID number of the process
			'pmem'
			,  # ratio of the process's resident set size to the physical memory
			'ppid',    # parent process ID
			'rss',     # resident set size
			'rtprio',  # realtime priority
			'sched',   # scheduling policy of the process
			'size',    # approximate total size
			'state',   # multi-character process state
			'sz',      # size in physical pages of the core image of the process
			'time',    # cumulative CPU time
			'vsz',     # virtual memory size of the process
			'wchan',   # name of the kernel function sleeping
			'command', # command with all its arguments as a string
		);
	}
	else {

		# all other unix : macos, hpux, ...
		return (
			'flags',    # flags associated with the process
			'nice',     # nice value. This ranges from 19 (nicest) to -20
			'pcpu',     # cpu utilization as a percentage
			'pid',      # process ID number of the process
			'pmem'
			,  # ratio of the process's resident set size to the physical memory
			'ppid',       # parent process ID
			'rss',        # resident set size
			'state',      # multi-character process state
			'time',       # cumulative CPU time
			'vsz',        # virtual memory size of the process
			'wchan',      # name of the kernel function sleeping
			'command',    # command with all its arguments as a string
		);
	}
}

{

	# parameters for ps command
	my %ps_keys;          # just a cache, do not use it

	# transform list in hash to access results
	sub get_ps_keys() {
		if ( !%ps_keys ) {
			my $indice = 0;
			foreach my $key ( get_ps_list() ) {
				$ps_keys{$key} = $indice++;
			}
		}
		return %ps_keys;
	}
}

###############################################################################
# get the list of running process
# should be runned only once by aquisition
sub _init_process() {

	# bsd format
	#my $info = "ps axlw";
	# standard options, specified format
	$ENV{'COLUMNS'} = 256;
	my $info = '/bin/ps ';
	if ( is_bsd() ) {
		$info .= DASH();
	}
	$info .= 'axo ' . join COMMA(), get_ps_list();

	my $r_tab = read_pipe($info);
	my $title = shift @{$r_tab};    # skip first line

	# check number of columns
	my @tab_title       = split_awk($title);
	my $nb_received_col = scalar @tab_title;
	my %keys            = get_ps_keys();
	my $nb_ask_col      = scalar keys %keys;
	if ( $nb_received_col != $nb_ask_col ) {

		# to fix it : ps axo ... > /devnull
		# should return an error message as
		# : ERROR: Unknown user-defined format specifier ...
		# but do not use warning : will display mesg before html header
		info(
"warning : ps configuration problem : asked $nb_ask_col fields, received $nb_received_col ( $title )"
		);
	}

	return $r_tab;
}
###############################################################################
# use a "private" var to store processus table
{
	my $tab_proc = _init_process();

	sub get_process() {
		return @{$tab_proc};
	}
}
###############################################################################
# allow access to ps output with a key, not an array number
sub get_ps_param($$) {
	my $ps_line = shift @_;
	my $param   = shift @_;    # ex : pid

	my %keys    = get_ps_keys();
	my $nb_keys = scalar keys %keys;

	if ( exists $keys{$param} ) {
		chomp $ps_line;
		my @tab = split_awk( $ps_line, $nb_keys );
		return $tab[ $keys{$param} ];
	}
	else {
		return;
	}
}
##########################################################
# search for a process in list (may be a regex)
sub search_process($) {
	my $searched_proc = shift @_;

	my @procs = get_process();
	foreach my $proc (@procs) {
		my $cmd = get_ps_param( $proc, 'command' );
		if ( $cmd =~ m/$searched_proc/ ) {
			return $cmd;
		}
	}
	return 0;
}
##########################################################
# call files
#
##########################################################
sub get_state_name($) {
	my $module = shift @_;

	my $call_suffix = '.call';
	return $ENV{'WEBMINSTAT_TEMP'} . SLASH() . $module . $call_suffix;
}

# maybe compute file name from module name ?
# read temp file
sub read_state($) {
	my $module = shift @_;

	# storage file name
	my $f_etat = get_state_name($module);

	my $r_tab = read_full_file($f_etat);
	if ($r_tab) {

		return @{$r_tab};
	}
	else {
		warning("no results from $f_etat");
		return;
	}
}
###############################################################################
# write temp file
sub write_state($@) {
	my $module = shift @_;
	my @info   = @_;         # info to be written

	# storage file name
	my $f_etat = get_state_name($module);

	if ( open my $fh_temp, '>', $f_etat ) {
		print {$fh_temp} @info;

		# seems to be problem with my_close here
		close $fh_temp or warning("can not close $f_etat : $ERRNO");
	}
	else {
		warning("can not write to temp file $f_etat : $ERRNO");
	}
	return;
}
##########################################################
# retrieve module name from module lib file
# ps : do not use get_module_name, already used in webmin
sub get_mod_name($) {
	my $file = shift @_;    # should be __FILE__

	my $module_name = basename($file);
	$module_name =~ s/-lib[.]pl//;
	return $module_name;
}
##########################################################
# a tool to store last aquisition date (in sysstats.pl)
# and get it in webmin page
my $lastrun = '/tmp/webminstats.lastrun';

#-------------------------------------------------------
sub set_lastrun() {
	touch($lastrun);
	return;
}

#-------------------------------------------------------
sub get_lastrun() {
	if ( -f $lastrun ) {

		# return mtime
		return ( stat $lastrun )[9];
	}
	else {
		return;
	}
}
##########################################################
# locking system to avoid launch an aquisition when
# the previous one is not ended (cf sysstats.pl)
my $lock = '/tmp/sysstats.lock';

#-------------------------------------------------------
sub is_locked() {

	if ( -f $lock ) {

		# get mtime in epoch
		my $date_lock = ( stat _ )[9];
		return $date_lock;
	}
	else {
		return;
	}
}

#-------------------------------------------------------
sub try_lock() {

	my $date_lock = is_locked();
	if ($date_lock) {
		my $date_current = time;    # current time in epoch
		if ( $date_current > ( $date_lock + 300 ) ) {

			# very old lock : > 5 minutes
			# can lock again
			touch($lock);
			return 1;
		}
		else {
			# already recent lock
			# can not lock
			return 0;
		}
	}
	else {
		# no lock : can lock
		touch($lock);
		return 1;
	}
}

#-------------------------------------------------------
sub unlock() {
	unlink $lock if ( is_locked() );
	return;
}
########################################################
# is used by mysql module
# in mysql-lib.pl and mysql.pl
# to set password for mysqladmin
sub set_mysql_passwd($) {
	my $passwd = shift @_;

	# sol 1 : password on command line
	# not secure : anyone can see the password with a ps command
	# return ' --password=' . quotemeta $passwd;

	# sol 2 : set password by environment variable
	# a little bit more secure
	$ENV{'MYSQL_PWD'} = $passwd;
	return q{};
}
###############################################################################
# return age of last change in a file in secondes versus current time
sub file_age_s($) {
	my $file = shift @_;

	if ( -e $file ) {
		my $file_epoch = ( stat _ )[9];
		my $cur_epoch  = time;

		return $cur_epoch - $file_epoch;
	}
	else {
		return;
	}
}
########################################################
# read module configuration (config file)
# can be called from
# - aquisition script, in this case, the config file is in the
#   current directory
# - webmin interface, in this case, a full config path is given
sub read_module_config($;$) {
	my $rh_config = shift @_;           # result hash
	my $path = shift @_ || 'config';    # path to config file

	# read config file
	my $ret = read_file( $path, $rh_config );

	if ($ret) {

		# check integrity
		my @tests = ( 'name', 'version' );
		foreach my $test (@tests) {
			if ( !exists $rh_config->{$test} ) {
				$ret = 0;               # false
				warning("bad config file $path : $test not found");
				last;
			}
		}
	}

	return $ret;
}
########################################################
# return process time ( not "real" human time)
# cf man time
sub get_process_time() {
	return (times)[0];
}
########################################################
1;
