#!/usr/bin/env perl
###############################################################################
#    Webmin Sysstats
#    Copyright (C) 2002 by Eric Gerbier
#    Bug reports to: gerbier@users.sourceforge.net
#    $Id$
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

# main aquisition script :
# - to be launched by cron
# - must execute each module script

use strict;
use warnings;
use Cwd;
use English qw(-no_match_vars);
use File::Basename;    # dirname
use POSIX ':sys_wait_h';
use RRDs;

use FindBin;
use lib "$FindBin::Bin";

## no critic (ProhibitPackageVars)
use vars qw( $module_name );
## use critic

# uncomment the following line to set in debug mode
# $ENV{'DEBUG_MODE'} = 1;

#####################################################################################
# tell if we are in parallel mode or not (standard sequential mode)
sub is_parallel_mode($) {
	my $config = shift @_;

	my $mode = 0;
	if ( exists $ENV{'PARALLEL_MODE'} ) {

		# allow force mode (for tests)
		$mode = 1;
	}
	elsif ( is_debug_mode() ) {

		# it is not easy to read mixed output
		$mode = 0;
	}
	elsif ( exists $config->{'parallel_mode'} ) {
		$mode = $config->{'parallel_mode'};
	}
	else {
		$mode = 0;
	}
	debug('parallel mode') if ($mode);
	return $mode;
}
#####################################################################################
# return true if aquisition is in start mode
# and apply sample_rate test
sub is_running($$) {
	my $min              = shift @_;    # minute (in a hour) of aquisition start
	my $rh_module_config = shift @_;    # module configuration hash

	my $ret;                            # return code

	# first test for runstop (note 0 for run, 1 for stop !)
	if ( $rh_module_config->{'runstop'} ) {
		debug( 'skip : runstop=' . $rh_module_config->{'runstop'} );
		$ret = 0;
	}
	else {
		my $sample_rate = $rh_module_config->{'sample_rate'} || 1;

		# general and easy case
		if ( $sample_rate == 1 ) {
			debug("ok : sample_rate=$sample_rate");
			$ret = 1;
		}

	# test for period : make a modulus beetwen minutes and sample_rate frequency
		elsif ( $min % $sample_rate ) {
			debug("skip cron run : min=$min sample_rate=$sample_rate");
			$ret = 0;
		}
		else {
			debug("modulus ok min=$min sample_rate=$sample_rate");
			$ret = 1;
		}
	}
	return $ret;
}
###############################################################################
# get "cron" current minutes (used for sample rate test)
sub get_min_of_hour(@) {

	my ( $sec, $min ) = (@_)[ 0 .. 1 ];

	# sometimes the cron start a few seconds before the round minutes
	# so we have to round the minute to avoid missing data (sample rate != 1 )
	$min = ( ( $min + 1 ) % 60 ) if ( $sec > 55 );
	return $min;
}
########################################################
# classic execution : one processus execute each module
# one at a time,
sub execute_mono($$$) {
	my $alarm_time = shift @_;
	my $module     = shift @_;
	my $perlscript = shift @_;

	# pre-script
	if ( is_debug_mode() ) {
		debug("start execute $perlscript");
	}
	else {
		unset_warnings();
	}

	# now execute it with a security if too long
	# the alarm is on real (clock) elapsed time
	alarm $alarm_time;
	do "./$perlscript";
	alarm 0;

	if ($EVAL_ERROR) {
		if ( $EVAL_ERROR =~ m/timeout/ ) {    # see alarme sub message
			warning("timeout on $module after $alarm_time seconds");
		}
		elsif ( $EVAL_ERROR =~ m/refcnt: fd/ ) {

			# dummy problem with close
			# do not warn
		}
		else {

			# not a timeout : another problem
			# !! we can not trap any error message from aquisition
			# the only message is sent by the last die command
			warning("strange error : $EVAL_ERROR");
		}
	}    # eval test

	# post-script
	if ( is_debug_mode() ) {
		debug("end execute $perlscript");
	}
	else {
		set_warnings();
	}
	return;
}
########################################################
# the goal is to reduce the execution time
# by using more than one proc
# threads can not be used (because current directory problems)
# so just fork one new process by module
sub execute_parallel($$$) {
	my $alarm_time = shift @_;
	my $module     = shift @_;
	my $perlscript = shift @_;

	# now execute in parallel
	my $pid = fork;
	if ($pid) {

		# father
		debug("creation fils $pid pour $module");
	}
	elsif ( defined $pid ) {

		# son
		execute_mono( $alarm_time, $module, $perlscript );
		exit;
	}
	else {
		warning("probleme bizarre sur fork : $ERRNO");
	}
	return;
}
########################################################
# signal handler on alarm signal
sub alarme() {
	die "timeout on alarm signal for $ENV{'module_name'}\n";
}
########################################################
# get_time : by default process time
sub get_time() {
	return get_process_time();
}
########################################################
# a security to avoid rrdtool warnings
# on too close update on a database
# return 1 if stamp is too near (not ok)
# return 0 if stamp is 10 s aged or more (ok)
sub test_stamp($) {
	my $stamp = shift @_;    # stamp file name
	my $ret;
	if ( -e $stamp ) {

		# old algo : compare date stamp with script's begin's date
		# may be not accurate if the script duration is long
		# my $age = -M $stamp;
		# 0.00011 is the value for one 10s in a day scale
		#if ( $age > 0.00011) {

		# new algo : now compare stamp date with current date
		my $file_age_s = file_age_s($stamp);
		if ( $file_age_s > 10 ) {

			debug( 'find old stamp : ' . $file_age_s . ' s' );
			$ret = 0;
		}
		else {
			debug(
				'skipped : find a too recent stamp : ' . $file_age_s . ' s' );
			$ret = 1;
		}
	}
	else {

		# no stamp : should be a first run
		$ret = 0;
	}
	return $ret;
}
########################################################
# overload global configuration with module configuration if it exists
sub overload($$$) {
	my $key              = shift @_;    # config key
	my $rh_global_config = shift @_;    # webminstats config hash
	my $rh_module_config = shift @_;    # module config hash

	my $val;
	if ( exists $rh_module_config->{$key} ) {
		$val = $rh_module_config->{$key};
		debug("use module value for $key : $val");
	}
	elsif ( exists $rh_global_config->{$key} ) {
		$val = $rh_global_config->{$key};
		debug("use global value for $key : $val");
	}
	else {
		$val = 0;
		warning("no value for $key");
	}

	return $val;
}
########################################################
# ref : perldoc -f waitpid
sub wait_for_childs() {

	my $kid;
	do {
		$kid = waitpid -1, WNOHANG;
	} while $kid > 0;

	return;
}
########################################################
# go to the execution directory
sub init_dir() {

	# fill module_name for sysstats-aquisition-lib.pl
	# could not use abs_path because $0 is a symbolic link
	my $gen_dir = dirname($PROGRAM_NAME);
	## no critic (ProhibitEscapedMetacharacters)
	if ( $gen_dir =~ m/^\.(.*)/ ) {

		# started with relative path
		$gen_dir = getcwd() . $1;
	}
	$module_name = basename($gen_dir);    # should be sysstats

	# temporary directory is now set by sysstats-aquisition-lib.pl
	## no critic (RequireBarewordIncludes)
	require 'sysstats-aquisition-lib.pl';
	## use critic

	# change directory to module directory (/etc/webmin/sysstats/modules)
	my $module_dir = $gen_dir . '/modules';
	if ( !chdir $module_dir ) {
		my_die("can not change directory to $module_dir : $ERRNO!");
	}

	# transform in absolute path
	$module_dir = getcwd();
	debug(  'current directory : '
		  . $module_dir
		  . "; current user : $REAL_USER_ID" );

	# set default PATH for external command to system directories
	clean_env();
	return $module_dir;
}
########################################################
# set nice priority
sub init_nice($) {
	my $config = shift @_;

	my $nice = $config->{'nice'} || 0;

	# check value nice (from -10 to +10) to avoid problems
	$nice = ( $nice > 10 )  ? 10  : $nice;
	$nice = ( $nice < -10 ) ? -10 : $nice;

	# renice job to have more time accuracy (cron problem)
	setpriority 0, 0, $nice;
	return;
}
########################################################
# display operating system info
sub init_debug($) {
	my $parallel = shift @_;

	if ( is_debug_mode() ) {

		# current id is display in init_dir
		debug('parallel mode') if ($parallel);

		# todo : display version

		my $cmd_lsb = search_command('lsb_release');
		if ($cmd_lsb) {
			## no critic (ProhibitBacktickOperators)
			my @out = `$cmd_lsb -a`;

			debug("operating system : @out");
		}
		else {
			debug("operating system : $OSNAME");
		}
	}
	return;
}
########################################################
# at end of module execution
# check duration
sub stat_module($$$) {
	my $begin_mod    = shift @_;
	my $max_mod_time = shift @_;
	my $module       = shift @_;

	my $end_mod = get_time();

	# warning on module time ?
	# we test here the process time
	my $duree_mod = $end_mod - $begin_mod;
	if ( ( $max_mod_time != 0 ) and ( $duree_mod > $max_mod_time ) ) {
		my $txt = sprintf "module $module execute is too long : %.2f seconds",
		  $duree_mod;
		warning($txt);
	}
	elsif ( is_debug_mode() ) {
		my $txt = sprintf "module $module execute in : %.2f seconds",
		  $duree_mod;
		debug($txt);
	}

	# fill internal.rrd if exists
	my $rrdbase = 'internal.rrd';
	if ( -f $rrdbase ) {
		RRDs::update( $rrdbase, "N:$duree_mod" );
	}
	return;
}
########################################################
# at end of aquisition
# warning on total execute time
sub stat_fin($$) {
	my $debut    = shift @_;
	my $max_time = shift @_;

	my $fin   = get_time();
	my $duree = $fin - $debut;
	if ( ( $max_time != 0 ) and ( $duree > $max_time ) ) {
		my $txt = sprintf 'webminstats aquisition is too long : %.2f seconds',
		  $duree;
		warning($txt);
	}
	elsif ( is_debug_mode() ) {
		my $txt = sprintf 'webminstats aquisition in %.2f seconds', $duree;
		debug($txt);
	}
	return;
}
########################################################
# return reference date : begin of aquisition
sub init_date() {

	# date calls : get run's reference
	my $date_ref = time;
	my @date_deb = localtime $date_ref;
	my $SEP      = PERIOD();
	my $date_deb =
	    $date_deb[2]
	  . $SEP
	  . $date_deb[1]
	  . $SEP
	  . $date_deb[0];    # format hh:mm:ss
	debug("sysstats.pl run at $date_deb");
	return get_min_of_hour(@date_deb);
}
########################################################
#     main code
########################################################
$OUTPUT_AUTOFLUSH = 1;    # no buffering

my $module_dir = init_dir();

# read modules directory
## no critic (ProhibitNoisyQuotes)
my @modules = readdir_content('.');
## use critic

# need to read general config file for time-out
my %config;
read_file( '../config', \%config );

my $parallel = is_parallel_mode( \%config );
init_nice( \%config );
init_debug($parallel);

# ref for total script time
my $debut = get_time();
my $min   = init_date();

# test if an older instance is still running
# another way could be to search in process list
## no critic (ProhibitUnlessBlocks)
unless ( try_lock() ) {
	debug('find lock : still running aquisition');
	exit;
}

# set an alarm callback to stop module aquisition if too long (see below eval)
$SIG{ALRM} = \&alarme;

# max time for all acquisition
my $max_time = $config{'max_time'};

alarm $max_time if ($parallel);
$ENV{'module_name'} = 'main';

# loop on each module
MODULES: foreach my $module (@modules) {
	next if ( $module =~ /^[.]{1,2}$/ );    # skip . and ..
	                                        #
	     # the fastest way is to have a relative chdir
	     # but some users reports problems (and I had sometimes too)
	     # if ( ! chdir($module) )
	     # 	# need the chdir ('..') at end of foreach to work

	# the secure way is to have an absolute chdir
	## no critic (ProhibitNoisyQuotes)
	my $new_path = $module_dir . '/' . $module;
	next unless ( -d $new_path );

	debug("***** module = $module");

	## use critic
	if ( !chdir $new_path ) {
		warning("can not chdir to $new_path : $ERRNO!");
		next MODULES;
	}

	my $perlscript = get_module_script($module);
	if ( !-o $perlscript ) {
		warning("no perl script in module directory ${module_dir}/$module");
		analyse_alarm($module);
		next MODULES;
	}

	# module begin time
	my $begin_mod = get_time();

	# set name for libraries
	$ENV{'module_name'} = $module;

	# get module config
	my %module_config;
	read_module_config( \%module_config );

	# test for run
	next MODULES unless is_running( $min, \%module_config );

	# a stamp to avoid running script too often
	# and avoid rrdtool warning
	# stamp is set at end of module run
	my $stamp = 'date.tamp';
	if ( test_stamp($stamp) ) {
		analyse_alarm($module);
		next MODULES;
	}

	# use module configuration if exists
	# else get webminstats global config
	my $max_mod_time = overload( 'max_mod_time', \%config, \%module_config );
	my $alarm_time   = overload( 'alarm_time',   \%config, \%module_config );
	my $max_log_size = overload( 'max_log_size', \%config, \%module_config );

	set_max_log_size($max_log_size);

	if ($parallel) {
		execute_parallel( $alarm_time, $module, $perlscript );
	}
	else {
		execute_mono( $alarm_time, $module, $perlscript );
	}

	# set the stamp for next run
	touch($stamp);

	stat_module( $begin_mod, $max_mod_time, $module );

	# clean alarm if necessary (cf sysstats-alarm-lib)
	analyse_alarm($module);

	delete $ENV{'module'};
}    # foreach

if ($parallel) {
	wait_for_childs();
	alarm 0;
}

set_lastrun();

stat_fin( $debut, $max_time );
unlock();
